/**
 * @class Sb.PartialTemplate
 * Represents an HTML fragment template partial.
 * @constructor
 * @param {String/Array} html
 * The HTML fragment or an array of fragments that will make up the template partial.
 * An Array will be combined with join('').
 */
Ext.define('Sb.PartialTemplate', {
	extend: 'Object',

	/**
	 * @cfg {RegExp} re The regular expression used to match template partial variables.
	 */
	re: /\{([\w\-]+)\}/g,

	constructor: function (html) {
		if (Ext.isArray(html)) {
			html = html.join("");
		}
		this.html = html;
	},

	/**
	 * Returns an HTML fragment of this template with the specified values applied.
	 * @param {Object} values
	 * @return {String} The HTML fragment
	 */
	apply: function (values) {
		// The regular expression used to match <Sitebuilder:tpl></Sitebuilder:tpl>
		var tplRe = /<Sitebuilder:tpl\b[^>]*>((?:(?=([^<]+))\2|<(?!Sitebuilder:tpl\b[^>]*>))*?)<\/Sitebuilder:tpl>/;

		// The regular expression used to match <Sitebuilder:tpl for="name"
		var nameRe = /^<Sitebuilder:tpl\b[^>]*?for="(.*?)"/;

		var html = ['<Sitebuilder:tpl>', this.html, '</Sitebuilder:tpl>'].join('');
		var chunks = [], id = 0, tplMatch;

		while ((tplMatch = html.match(tplRe))) {
			var nameMatch = tplMatch[0].match(nameRe);
			var name = nameMatch && nameMatch[1] ? nameMatch[1] : '';

			chunks.push({
				id: id,
				name: name,
				body: tplMatch[1]
			});
			html = html.replace(tplMatch[0], '{__tpl'+ id + '}');
			id++;
		}

		return chunks[chunks.length - 1].body.replace(this.re, function () {
			return this._applySubPartial(arguments[1], values, chunks);
		}.createDelegate(this));
	},

	/**
	 * @private Internal method for chunks of template partial with the specified values applied
	 *
	 * @param {String} name Name of partial variable
	 * @param {Object} values The tempalte partial values
	 * @param {Array} chunks Array of partial chunks
	 * @return string The HTML fragment
	 */
	_applySubPartial: function (name, values, chunks) {
		if ('__tpl' == name.substr(0, 5)) {
			var id = name.substr(5);

			if (!chunks[id] || !values[chunks[id].name]) {
				return '';
			}

			if (Ext.isArray(values[chunks[id].name])) {
				var ret = '';
				Ext.each(values[chunks[id].name], function (v) {
					ret += chunks[id].body.replace(this.re, function () {
						return this._applySubPartial(arguments[1], v, chunks);
					}.createDelegate(this));
				}, this);
				return ret;
			}

			return chunks[id].body.replace(this.re, function () {
				return this._applySubPartial(arguments[1], values[chunks[id].name], chunks);
			}.createDelegate(this));
		}

		return values[name] || '';
	},

	/**
	 * Applies the supplied <code>values</code> to the template and overwrites
	 * the content of <code>el</code> with the new node(s).
	 * @param {mixed} el The context element
	 * @param {Object} values The template values.
	 */
	update: function (el, values) {
		el = Ext.getDom(el);
		el.innerHTML = this.apply(values);
	},

	/**
	 * Applies the supplied <code>values</code> to the template and appends
	 * the new node(s) to the specified <code>el</code>.
	 * @param {mixed} el The context element
	 * @param {Object} values The template values.
	 */
	append: function (el, values) {
		el = Ext.getDom(el);
		Ext.DomHelper.insertHtml('beforeEnd', el, this.apply(values));
	}
});
