/**
 * @class Sb.Utils
 * <p>Contains a bunch of usable utils.</p>
 * @singleton
 */
Ext.define('Sb.Utils', {
	singleton: true,

	/**
	 * Generate uuid
	 * @return {String} uuid
	 */
	generateUuid: function () {
		var chars = '0123456789abcdef'.split(''), uuid = [], r, i;
		// rfc4122 requires these characters
		uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
		uuid[14] = '4';

		// Fill in random data. At i==19 set the high bits of clock sequence as
		// per rfc4122, sec. 4.1.5
		for (i = 0; i < 36; i++) {
			if (!uuid[i]) {
				r = 0 | Math.random() * 16;
				uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
			}
		}
		return uuid.join('');
	},

	/**
	 * Generate uniquie identity for page instance
	 * @return {String}
	 * @see Base_Utils_String::getShortHash()
	 */
	getShortHash: function () {
		var template = '1234567890abcdefghijklmnopqrstuvwxyz';
		var rndstring = '';
		var b = 0, a;
		for (a = 0; a <= 10; a++) {
			b = Math.floor(Math.random() * template.length);
			rndstring += template.substr(b, 1);
		}
		return rndstring;
	},

	/**
	 * Generates random number between 0 and "to" param inclusive.
	 * When to is array, it returns random value from it.
	 *
	 * @param {Number|Array|Ext.util.MixedCollection} to Upper limit
	 * @return {Number|Object}
	 */
	random: function (to) {
		if (Ext.isArray(to)) {
			return to[this.randomIndex(to)];
		}
		if (to instanceof Ext.data.ArrayStore) {
			return to.getAt(this.randomIndex(to.data));
		}
		return Math.floor(Math.random()*to);
	},

	/**
	 * Returns random index from input array.
	 *
	 * @param {Array|Ext.util.MixedCollection} arrayValue
	 * @return {Number}
	 */
	randomIndex: function (arrayValue) {
		return this.random(arrayValue.length);
	},

	/**
	 * Copy of PHP function array_values
	 * Returns all the values from the input array and indexes the array numerically.
	 *
	 * @param {Array|Object} array
	 * @return {Array}
	 */
	arrayValues: function (array) {
		var result = [];

		Ext.each(array, function (value) {
			if (value === undefined) {
				return true;
			}
			result.push(value);
		});

		return result;
	},

	/**
	 * Returns regexp for validating image extensions
	 *
	 * @return {RegExp}
	 */
	getImageValidExtensionsRegex: function () {
		return new RegExp('\\.(' + Sb.Config.imageValidExtensions.join('|') + ')$', 'i');
	},

	/**
	 * Check image original size and returns it in callback
	 * @param {String} url Url of image
	 * @param {Function} callback
	 */
	getImageOriginalSize: function (url, callback) {
		var image = new Ext.Component({
			xtype: 'box',
			autoEl: {
				tag: 'img',
				src: url
			},
			renderTo: Ext.getBody(),
			style: {
				'text-indent': '-9999px',
				'visibility': 'hidden'
			}
		});

		image.el.on('load', function () {
			var size = image.el.getSize();
			image.destroy();
			callback(size);
		});
	},

	/**
	 * Calculate second proportional size of image by first size
	 * @param {String} dimension 'width' or 'height'
	 * @param {Number} value second size in px
	 * @param {Object} originalSize
	 * @return {Number}
	 */
	getProportionalSize: function (dimension, value, originalSize) {
		var secondDimension = (dimension == 'width') ? 'height': 'width';

		return (originalSize[dimension] > 0)
			? Math.round(value * originalSize[secondDimension] / originalSize[dimension])
			: 0;
	},

	isProportional: function (width, height, originalSize) {
		var ratio = (height > 0) ? width / height : 0,
			originalRatio = (originalSize.height > 0) ? originalSize.width / originalSize.height : 0;

		return (ratio == originalRatio);
	},

	/**
	 * Returns param value from query string
	 * @param {String} Param name
	 * @return {String}
	 */
	getQueryParamValue: function (param) {
		var query = self.location.search || self.location.hash;
		if (query) {
			if (/\?/.test(query)) {
				query = query.split('?')[1];
			}
			var i, pairs = query.split('&');
			for (i = 0; i < pairs.length; i++) {
				if (pairs[i].substring(0, pairs[i].indexOf('=')) == param) {
					return decodeURIComponent(pairs[i].substring((pairs[i].indexOf('=') + 1)));
				}
			}
		}
		return '';
	},

	/**
	 * Compare 2 specified versions, starting from left to right.
	 * @param {String} current The current version to compare to
	 * @param {String} target The target version to compare to
	 * @return {Number} Returns -1 if the current version is smaller than the target version, 1 if greater, and 0 if they're equivalent
	 */
	compareVersions: function (current, target) {
		var currentValue, targetValue, i;

		current = current.split('.');
		target = target.split('.');

		for (i = 0; i < Math.max(current.length, target.length); i++) {
			currentValue = parseInt(current[i] || 0, 10);
			targetValue = parseInt(target[i] || 0, 10);

			if (currentValue < targetValue) {
				return -1;
			}
			if (currentValue > targetValue) {
				return 1;
			}
		}

		return 0;
	},

	/**
	 * Return type of file by it's extension
	 * @param {String} extension Extension like "txt"
	 * @return {String} extension
	 */
	getDocumentTypeByExtension: function (extension) {
		var result = 'default', extensions = {
			'access': ['accdb', 'mdb', 'mdf'],
			'acrobat': ['pdf'],
			'archive': ['iso', 'tar', 'bz2', 'gz', '7z', 'arc', 'arj', 'cab', 'rar', 'sfx', 'tar.gz', 'tgz', 'zip'],
			'audio': ['4mp', 'aif', 'iff', 'm3u', 'm4a', 'mid', 'mp3', 'mpa', 'ra', 'wav', 'wma'],
			'excel': ['xls', 'xlsx', 'xlsm', 'xlsb'],
			'html': ['htm', 'html'],
			'image': ['001', 'bmp', 'gif', 'jpg', 'jpeg', 'png', 'psd', 'pspimage', 'thm', 'tif', 'yuv', 'tiff', 'tif', 'xcf'],
			'powerpoint': ['ppt', 'pptx', 'pps', 'ppsx'],
			'rtf': ['rtf'],
			'script': ['asp', 'aspx', 'css', 'chm', 'js', 'json', 'jsp', 'php', 'phtml', 'shtml', 'xhtm', 'xhtml', 'wml'],
			'text': ['1st', 'log', 'msg', 'pages', 'txt', 'wpd', 'wps'],
			'video': ['3g2', '3gp', 'asf', 'asx', 'avi', 'flv', 'mov', 'mp4', 'mpg', 'rm', 'swf', 'vob', 'wmv'],
			'visio': ['vsd', 'vss', 'vst', 'vsw', 'vdx', 'vsx', 'vtx', 'vsl'],
			'word': ['doc', 'docx'],
			'xml': ['xml', 'xsl', 'xslt', 'rss', 'atom']
		};

		Ext.iterate(extensions, function (type, typeExtensions) {
			if (typeExtensions.indexOf(extension) !== -1) {
				result = type;
				return false;
			}
		});

		return result;
	},

	/**
	 * Return native application of file by it's extension
	 * @param {String} extension Extension like "txt"
	 * @return {String} application name or extension by default
	 */
	getDocumentAppByExtension: function (extension) {
		var localeKey = extension + 'Application',
			locale = Sb.Locale.get(localeKey);

		return (locale == '[' + localeKey + ']') ? Ext.util.Format.capitalize(extension) : locale;
	},

	/**
	 * Add anti cache parameters to source
	 * @param {String} source Path to resource
	 * @param {Boolean} [always] Update anticache parameter every time or by build only
	 * @return {String} Anticached path to resource
	 */
	antiCache: function (source, always) {
		return source + '?' + (always ? (new Date()).getTime() :
			Sb.Config.general.version + '_' + Sb.Config.general.build);
	},

	swapLeftAndRight: function (value) {
		var pairs = [
				['Left', 'Right'],
				['left', 'right'],
				['column1', 'column2'],
				['First', 'Second'],
				['sidebarFirst', 'sidebarSecond']
			],
			result = value;

		Ext.each(pairs, function (pair) {
			var index = pair.indexOf(value);
			if (index != -1) {
				result = pair[Math.abs(index - 1)];
				return false;
			}
		});

		return result;
	},

	swapCssCornersValues: function (value) {
		return value.replace(/(\S+) (\S+) (\S+) (\S+)/, '$2 $1 $4 $3');
	},

	/**
	 * Helper to get background type
	 * @param {String} value
	 * @return {Object} background type
	 */
	getBackgroundType: function (value) {
		// Transparent or none
		if (Ext.isEmpty(value) || value === 'transparent') {
			return {value: 'none', isCustom: false};
		}

		// Gradient
		if (/^background\d+-(black|white)$/.test(value)) {
			return {value: 'gradient', isCustom: false};
		}

		// Header as background
		if (value.indexOf('.') !== -1) {
			if (value.indexOf('builtin/') !== -1) {
				return {value: 'image', isCustom: false};
			}
			if (value.indexOf('Header/') !== -1) {
				return {value: 'image', isCustom: true};
			}
		}

		// Image as background
		if (/^background\d+$/.test(value)) {
			return {value: 'pattern', isCustom: false};
		}

		// Applying images from preset
		if (/^[\w|\-]+$/.test(value)) {
			return {value: 'image', isCustom: false};
		}

		// Fallback
		return {value: 'pattern', isCustom: true};
	},

	getWidgetButtonCfg: function (typeName, title) {
		return {
			cls: 'sb-widgets-item-button sb-widgets-item-' + typeName,
			cn: [{
				cls: 'sb-widgets-item-wrap',
				cn: {
					cls: 'sb-widgets-item',
					cn: [{
						cls: 'sb-widgets-item-icon',
						cn: {
							cls: 'sb-widgets-item-icon-inner'
						}
					}, {
						cls: 'sb-widgets-item-content',
						cn: {
							tag: 'span',
							html: title
						}
					}]
				}
			}]
		};
	},

	isInnerShadow: function (cssValue) {
		return cssValue.indexOf('inset') > -1;
	},

	setBackgroundImage: function (el, imageUrl, alpha, color) {
		var image = new window.Image();
		image.onload = function() {
			var canvas = document.createElement('canvas');
			canvas.width = this.width;
			canvas.height = this.height;

			var context = canvas.getContext('2d');

			if (color != 'transparent') {
				context.fillStyle = color;
				context.fillRect(0, 0, canvas.width, canvas.height);
			}

			context.drawImage(image, 0, 0);

			if (alpha < 1) {
				if (canvas.toDataURL() != image.src) {
					image.src = canvas.toDataURL();
				}
				context.clearRect(0, 0, canvas.width, canvas.height);
				context.globalAlpha = alpha;
				context.drawImage(image, 0, 0);
			}

			Ext.get(el).setStyle('backgroundImage', 'url("' + canvas.toDataURL() + '")');

			Ext.removeNode(image);
			Ext.removeNode(canvas);
		};
		image.src = imageUrl;
	}
});