/**
 * @class Sb.auxilaryModel.Image
 * @extends Sb.model.Node
 * Image get/set facade (DOM as source).
 * @constructor
 * @param {Object} config Configuration for image
 */
Ext.define('Sb.auxilaryModel.Image', {
	extend: 'Sb.model.Node',

	constructor: function (config) {
		config = config || {};

		/**
		 * Current image dom element to operate with.
		 *
		 * @type Ext.Element
		 * @property el
		 */
		this.el = {};

		/**
		 * Can we update element and it`s DOM?
		 *
		 * @type Boolean
		 * @property canUpdateEl
		 */
		this.canUpdateEl = true;

		this.addEvents(
			/**
			 * @event alignchange
			 * Fires when image align setted or changed.
			 * @param {Sb.auxilaryModel.Image} this This image.
			 */
			'alignchange',
			/**
			 * @event heightchange
			 * Fires when image height setted or changed.
			 * @param {Sb.auxilaryModel.Image} this This image.
			 */
			'heightchange'
		);
		Sb.auxilaryModel.Image.superclass.constructor.call(this, config);
	},

	/**
	 * Sets current image`s DOM element as Ext`s Element.
	 *
	 * @param {Object} domEl DOM element link or query
	 */
	setElement: function (domEl) {
		var elCandidate = Ext.get(domEl);
		if (null == elCandidate || !elCandidate.is('img')) {
			return false;
		}
		this.el = elCandidate;
		this.canUpdateEl	= false;

		var width = this.el.getWidth(),
			height = this.el.getHeight();

		this.unserialize({
			sourceUrl			: this.el.getAttribute('data-mce-src') || this.el.getAttribute('src'),
			width				: width,
			height				: height,
			alignTo				: Sb.utils.Tinymce.getAlignment(this.el),
			alternativeText		: this.el.getAttribute('title') || this.el.getAttribute('alt'),
			caption				: this.getCaptionStateFromEl(),
			isProportional		: Sb.Utils.isProportional(
				this.el.getWidth(), this.el.getHeight(), this.get('originalSize')
			)
		});

		this.canUpdateEl = true;
		this.setAlignment(this.get("alignTo"));

		this.fireEvent('load', this);
		return true;
	},

	/**
	 * Fires on changing of single property change to allow us fire up more abstract change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		var value = this.data[key];

		if ('sourceUrl' == key) {
			var rawSourceFileName	= decodeURIComponent(value.split('/').pop());
			rawSourceFileName		= rawSourceFileName.replace(/=/g, '%');
			var sourceFileName		= decodeURIComponent(rawSourceFileName);
			sourceFileName			= sourceFileName.replace('\\', '');
			this.unserialize({
				rawSourceFileName	: rawSourceFileName,
				sourceFileName		: sourceFileName
			});
		} else if ('alignTo' == key) {
			value = value.toLowerCase();
			if (!(/^(left|right|center)$/).test(value)) {
				value = 'none';
			}
			this.data[key] = value;
		} else {
			this.set(key, value);
		}
		this.updateDomElement(key, value);
		if ('alignTo' == key) {
			this.fireEvent('alignchange', this);
		}
	},

	/**
	 * Updates DOM with values. Also updates editor-specific attributes.
	 *
	 * @param {String} key
	 * @param {String|Number} value
	 */
	updateDomElement: function (key, value) {
		if (!this.canUpdateEl) {
			return;
		}
		var el = this.el;
		if (key == 'sourceUrl') {
			el.set({src: value, 'data-mce-src': value});
		} else if (key == 'width') {
			el.set({width: value});
		} else if (key == 'height') {
			el.set({height: value});
			this.fireEvent('heightchange', this);
		} else if (key == 'alignTo') {
			this.setAlignment(value);
		} else if (key == 'alternativeText') {
			el.set({alt: value});
			el.set({title: value});
		} else if (key == 'caption') {
			this.setCaptionStateToEl(value);
		}
	},

	/**
	 * Set alignment of selected element
	 *
	 * @param {String} value
	 */
	setAlignment: function (value) {
		var el		= this.el;
		var style	= {
			'float'		: '',
			marginLeft	: '',
			marginRight	: '',
			display		: ''
		};
		if ('left' == value) {
			style['float']		= 'left';
			style.marginRight	= '20px';
		} else if ('right' == value) {
			style['float']		= 'right';
			style.marginLeft	= '20px';
		} else if ('center' == value) {
			style.marginLeft	= 'auto';
			style.marginRight	= 'auto';
			style.display		= 'block';
		}
		el.setStyle(style);
		Sb.utils.Tinymce.cacheStyleInEditor(el);
	},

	/**
	 * Selects image in editor
	 */
	selectInEditor: function () {
		Sb.utils.Tinymce.selectInEditor(this.el);
	},

	/**
	 * Removes image from editor
	 */
	removeFromEditor: function () {
		Sb.utils.Tinymce.removeFromEditor(this.el);
	},

	/**
	 * Calculate second proportional size of image by first size
	 * @param {String} dimension 'width' or 'height'
	 * @param {Number} value second size in px
	 * @return {Number}
	 */
	getProportionalSize: function (dimension, value) {
		return Sb.Utils.getProportionalSize(dimension, value, this.get('originalSize'));
	},

	/**
	 * Update dimension size depends of proportions constrain
	 * @param {String} dimension 'width' or 'height'
	 */
	updateDimension: function (dimension) {
		var secondDimension, value;

		if (this.get('isProportional')) {
			secondDimension = (dimension == 'width') ? 'height': 'width';
			value = this.getProportionalSize(secondDimension, this.get(secondDimension));
		} else {
			value = this.get(dimension);
		}

		this.set(dimension, value);
	},

	getCaptionStateFromEl: function () {
		var result,
			classes = [
				Sb.auxilaryModel.Image.captionUnderImage,
				Sb.auxilaryModel.Image.captionOverImage,
				Sb.auxilaryModel.Image.captionNotShow
			];

		Ext.each(classes, function (cls) {
			if (this.el.hasClass(cls)) {
				result = cls;
				return false;
			}
		}, this);

		if (!result) {
			result = Sb.auxilaryModel.Image.captionOverImage;
			this.setCaptionStateToEl(result);
		}

		return result;
	},

	setCaptionStateToEl: function (value) {
		var classes = [
			Sb.auxilaryModel.Image.captionUnderImage,
			Sb.auxilaryModel.Image.captionOverImage,
			Sb.auxilaryModel.Image.captionNotShow
		];
		Ext.each(classes, function (cls) {
			this.el.removeClass(cls);
		}, this);
		this.el.addClass(value);
	}
});

Sb.auxilaryModel.Image.captionUnderImage = 'caption-under-image';
Sb.auxilaryModel.Image.captionOverImage = 'caption-over-image';
Sb.auxilaryModel.Image.captionNotShow = 'caption-not-show';
