/**
 * @class Sb.dashboard.Manager
 * @extends Ext.util.Observable
 * Base class for dashboard tiles
 */
Ext.define('Sb.dashboard.Manager', {
	extend: 'Ext.util.Observable',

	modelsMap: {
		disqus: Sb.dashboard.model.Disqus,
		ga: Sb.dashboard.model.GoogleAnalytics,
		ecwid: Sb.dashboard.model.Ecwid,
		upsell: Sb.dashboard.model.Upsell
	},

	modelsCache: {},

	/**
	 * {Ext.KeyMap}
	 */
	keysMap: undefined,

	/**
	 * Dashboard window instance
	 */
	window: null,

	constructor: function (config) {
		this.keysMap = new Ext.KeyMap(Ext.getBody(), [{
			key: Ext.EventObject.ESC,
			scope: this,
			fn: function () {
				if (this.window) {
					this.window.closeHandler();
				}
			}
		}]);

		Sb.dashboard.Manager.superclass.constructor.call(this, config);
	},

	/**
	 * @param modelAlias
	 * @return Sb.dashboard.Model
	 */
	getModel: function (modelAlias) {
		if (!this.modelsCache[modelAlias]) {
			this.modelsCache[modelAlias] = new this.modelsMap[modelAlias]();
		}
		return this.modelsCache[modelAlias];
	},

	getOverviewTabConfig: function () {
		var config = {
			items: []
		};

		if (Sb.Config.googleAnalyticsEnabled) {
			config.items.push({
				manager: this,
				xtype: 'sb.dashboard.tile.ga',
				colspan: 2
			});
		}

		if (this.getModel('disqus').isEnabled()) {
			config.items.push({
				manager: this,
				xtype: 'sb.dashboard.tile.disqus'
			});
		}

		if (this.getModel('ecwid').isEnabled()) {
			config.items.push({
				manager: this,
				xtype: 'sb.dashboard.tile.ecwid'
			});
		}

		if (this.getModel('upsell').isEnabled()) {
			config.items.push({
				manager: this,
				xtype: 'sb.dashboard.tile.upsell'
			});
		}

		return config;
	},

	getNotificationTabConfig: function () {
		var config = {
			items: {
				xtype: 'tabpanel',
				activeTab: 0,
				baseCls: 'sb-dashboard-tab-panel',
				ctCls: 'sb-dashboard-notification',
				autoHeight: true,
				items: [],
				listeners: {
					afterrender: function (tabpanel) {
						var headerWidth = tabpanel.header.dom.offsetWidth,
							tabs = tabpanel.strip.query('li:not(.x-tab-edge)'),
							width = Math.floor(headerWidth / tabs.length) + 'px';

						Ext.each(tabs, function (tab) {
							tab.style.width = width;
						});
					}
				}
			}
		};

		if (this.getModel('disqus').isEnabled()) {
			config.items.items.push({
				manager: this,
				tabCls: 'sb-dashboard-tab sb-dashboard-tab-comments',
				title: '<div class="sb-dashboard-tab-text"><h1>' + Sb.Locale.get('comments') + '</h1></div>',
				ref: 'commentsTab',
				xtype: 'sb.dashboard.tile.disqusdetail'
			});
		}

		if (this.getModel('ecwid').isEnabled()) {
			config.items.items.push({
				manager: this,
				tabCls: 'sb-dashboard-tab sb-dashboard-tab-orders',
				title: '<div class="sb-dashboard-tab-text"><h1>' + Sb.Locale.get('orders') + '</h1></div>',
				ref: 'ordersTab',
				xtype: 'sb.dashboard.tile.ecwiddetail'
			});
		}

		config.items.items[0].tabCls += ' sb-dashboard-tab-first';
		config.items.items[config.items.items.length - 1].tabCls += ' sb-dashboard-tab-last';

		return config;
	},

	/**
	 * @param {Sb.dashboard.Window} window
	 */
	createTiles: function (window) {
		var windowTabsConfig = {
			overviewTab: this.getOverviewTabConfig()
		};

		if (Sb.Config.googleAnalyticsEnabled) {
			windowTabsConfig.googleAnalyticsTab = {
				items: {
					manager: this,
					xtype: 'sb.dashboard.tile.gadetailed'
				}
			};
		}

		if (this.getModel('disqus').isEnabled() || this.getModel('ecwid').isEnabled()) {
			windowTabsConfig.notificationTab = this.getNotificationTabConfig();
		}

		if (Sb.Config.isDashboardExternalServicesEnabled) {
			windowTabsConfig.externalServicesTab = {
				items: {
					manager: this,
					xtype: 'sb.dashboard.tile.externalservices'
				}
			};
		}

		Ext.iterate(windowTabsConfig, function (tab, config) {
			window.tabs[tab].add(config.items);
		});
	},

	/**
	 * Create if not exist dashboard window instance and returns it
	 *
	 * @return {Sb.dashboard.Window}
	 */
	getWindow: function () {
		if (!this.window || this.window.isDestroyed) {
			this.window = new Sb.dashboard.Window({
				showAnalyticsTab: Sb.Config.googleAnalyticsEnabled,
				showNotificationTab: this.getModel('disqus').isEnabled() || this.getModel('ecwid').isEnabled(),
				showExternalServicesTab: Sb.Config.isDashboardExternalServicesEnabled
			});
			this.createTiles(this.window);
		}
		return this.window;
	},

	/**
	 * Returns true if dashboard is available
	 *
	 * @returns {boolean}
	 */
	isAvailable: function () {
		var win = this.getWindow();
		return this.getOverviewTabConfig().items.length > 0 || win.showAnalyticsTab || win.showNotificationTab ||
			win.showExternalServicesTab;
	},

	/**
	 * Show dashboard window
	 */
	showWindow: function () {
		return this.getWindow().show();
	}

});
