/**
 * Edit image dialog for text widget
 *
 * @class Sb.editor.ImageEditDialog
 * @extends Sb.Window
 */
Ext.define('Sb.editor.ImageEditDialog', {
	extend: 'Sb.editor.ModelEditDialog',

	uploadUrl: Sb.Config.baseUrl + '/Upload/Image/',
	layout: 'column',
	title: Sb.Locale.get('imageEditDialogEdit'),
	cls: 'sb-dialog-image',
	hiddenAltText: false,
	editDialogSizeInPixels: Sb.Locale.get('imageEditDialogSizeInPixels'),
	editDialogSizeReset: Sb.Locale.get('imageEditDialogSizeReset'),
	editDialogSizeConstrainProportions: Sb.Locale.get('imageEditDialogSizeConstrainProportions'),
	editDialogChange: Sb.Locale.get('imageEditDialogChange'),
	editDialogTextWrapping: Sb.Locale.get('imageEditDialogTextWrapping'),
	editDialogAlternativeTextLabel: Sb.Locale.get('imageEditDialogAlternativeTextLabel'),
	editDialogAlternativeTextDescription: Sb.Locale.get('imageEditDialogAlternativeTextDescription'),
	editDialogPreview: Sb.Locale.get('imageEditDialogPreview'),
	editDialogChangeWarningInvalidType: Sb.Locale.get('imageEditDialogChangeWarningInvalidType'),
	uploadingImage: Sb.Locale.get('uploadingImage'),
	uploadValidRegex: Sb.Utils.getImageValidExtensionsRegex(),

	constructor: function (config) {
		config = Ext.apply({
			/**
			 * @cfg {Object} domEl DOM node of edited image.
			 */
			domEl				: null,
			/**
			 * @cfg {Boolean} isUploaded When image is uploaded and not changed
			 */
			isUploaded			: false,
			/**
			 * @cfg {Function} afterHeightChange Fires after height changed
			 */
			afterHeightChange: Ext.emptyFn,
			modelClass: Sb.auxilaryModel.Image
		}, config);

		Sb.editor.ImageEditDialog.superclass.constructor.call(this, config);
	},

	initComponent: function () {
		var columnDefault = {
			columnWidth	: 0.5,
			border		: false
		};
		var leftColumn	= new Ext.Container(
			Ext.apply({
				cls			: 'sb-dialog-image-block1'
			}, columnDefault)
		);
		var rightColumn	= new Ext.form.FormPanel(
			Ext.apply({
				cls			: 'sb-dialog-image-block2',
				labelAlign	: 'top'
			}, columnDefault)
		);
		this.items		= [leftColumn, rightColumn];

		// Left column controls
		/**
		 * Preview label, that shows us file name
		 * @type Ext.form.Label
		 * @property previewLbl
		 */
		this.previewLbl	= new Ext.form.Label();
		var previewCtn	= new Ext.Container({
			cls			: 'sb-dialog-image-preview'
		});
		/**
		 * Preview image, that will shows us current image or uploaded image
		 * @type Ext.BoxComponent
		 * @property previewImg
		 */
		this.previewImg = this.createPreview();
		previewCtn.add(this.previewImg);

		var sizeInPxLbl	= new Ext.form.Label({
			text		: this.editDialogSizeInPixels
		});

		var dimensionDefault = {
			allowDecimals: false,
			allowBlank: false,
			minValue: 10,
			maxLength: 4,
			width: 43
		};

		/**
		 * Image width field
		 * @type Ext.form.TextField
		 * @property widthFld
		 */
		this.widthFld	= new Ext.form.NumberField(
			Ext.apply({
				name: 'width'
			}, dimensionDefault)
		);

		/**
		 * Image height field
		 * @type Ext.form.TextField
		 * @property heightFld
		 */
		this.heightFld	= new Ext.form.NumberField(
			Ext.apply({
				name: 'height'
			}, dimensionDefault)
		);

		this.resetButton = new Ext.Button({
			text: this.editDialogSizeReset,
			cls: 'sb-image-size-control-reset'
		});

		this.constrainProportionsCheckbox = new Ext.form.Checkbox({
			hideLabel: true,
			boxLabel: this.editDialogSizeConstrainProportions
		});

		var uploadFrm	= new Ext.form.FormPanel({
			fileUpload	: true,
			border		: false,
			hideLabels	: true
		});
		/**
		 * Change image button
		 * @type Ext.ux.form.FileUploadField
		 * @property uploadFld
		 */
		this.uploadFld	= new Ext.ux.form.FileUploadField({
			regex		: this.uploadValidRegex,
			name		: 'image',
			msgTarget	: 'under',
			buttonText	: this.editDialogChange,
			buttonOnly	: true
		});
		uploadFrm.add(this.uploadFld);
		var sizeFields = {
			layout: 'column',
			border: false,
			items: [this.widthFld, {
				border: false,
				cls: 'x',
				html: 'x'
			}, this.heightFld, this.resetButton]
		};

		var sizeControl = {
			xtype: 'container',
			fieldLabel: this.editDialogSizeInPixels,
			cls: 'sb-image-size-control',
			items: [sizeFields, this.constrainProportionsCheckbox]
		};

		leftColumn.add([this.previewLbl, previewCtn, sizeInPxLbl, sizeControl, uploadFrm]);

		// Right column controls
		/**
		 * Change image button
		 * @type Ext.ux.form.FileUploadField
		 * @property uploadFld
		 */
		this.alignCtl	= new Ext.form.RadioGroup({
			cls				: 'sb-dialog-image-position',
			fieldLabel		: this.editDialogTextWrapping,
			labelSeparator	: '',
			hideLabels		: true,
			items			: []
		});
		/**
		 * Possible align value
		 *
		 * @type Array
		 * @property linkStates
		 */
		this.alignValues	= ['left', 'right', 'center'];
		Ext.each(this.alignValues, function (alignValue) {
			this.alignCtl.items.push(new Ext.form.Radio({
				inputValue		: alignValue,
				name			: 'alignValue',
				hideLabel		: true,
				width			: 71,
				height			: 61,
				itemCls			: 'sb-dialog-image-position-' + alignValue
			}));
		}, this);

		this.altTextFld		= new Ext.form.TextArea({
			fieldLabel		: this.editDialogAlternativeTextLabel,
			itemCls			: 'sb-dialog-image-alt-text',
			anchor			: '100%',
			height			: 44,
			hidden			: this.hiddenAltText,
			maxLength		: 1000,
			msgTarget		: 'under'
		});
		this.defaultButton	= this.altTextFld;
		this.captionOptionsFld = new Ext.form.ComboBox({
			fieldLabel: Sb.Locale.get('imageDialogCaptionOptions'),
			anchor: '100%',
			mode: 'local',
			displayField: 'display',
			valueField: 'value',
			hidden: this.hiddenAltText,
			store: new Ext.data.ArrayStore({
				fields: ['value', 'display'],
				data: [
					[Sb.auxilaryModel.Image.captionUnderImage, Sb.Locale.get('imageCaptionUnderImage')],
					[Sb.auxilaryModel.Image.captionOverImage, Sb.Locale.get('imageCaptionOverImage')],
					[Sb.auxilaryModel.Image.captionNotShow, Sb.Locale.get('imageCaptionNotShow')]
				]
			})
		});
		var altTextLbl		= new Ext.Container({
			cls				: 'sb-editor-box-description',
			html			: this.editDialogAlternativeTextDescription,
			hidden			: this.hiddenAltText
		});

		rightColumn.add([
			this.alignCtl, this.altTextFld, this.captionOptionsFld, altTextLbl
		]);

		if (Ext.isArray(this.rightColumnItems)) {
			rightColumn.add(this.rightColumnItems);
		}
		Sb.editor.ImageEditDialog.superclass.initComponent.call(this);
	},

	createPreview: function () {
		return new Ext.BoxComponent({
			autoEl: {
				tag: 'IMG',
				src: Ext.BLANK_IMAGE_URL
			}
		});
	},

	updatePreview: function (src) {
		this.previewImg.el.set({src: src});
	},

	/**
	 * Init events
	 */
	initEvents: function () {
		Sb.editor.ImageEditDialog.superclass.initEvents.call(this);

		this.model.setElement(this.domEl);
		this.memento = this.model.saveToMemento();

		this.model.on('heightchange', this.afterHeightChange, this);
		this.model.on('alignchange', this.afterAlignChange, this);

		this.widthFld.on('valid', this.onSizeChange, this);
		this.widthFld.on('blur', this.onSizeBlur, this);
		this.heightFld.on('valid', this.onSizeChange, this);
		this.heightFld.on('blur', this.onSizeBlur, this);
		this.resetButton.on('click', this.onSizeReset, this);
		this.uploadFld.on('fileselected', this.onFileSelect, this);
		this.alignCtl.on('change', this.onPropertyChange, this);
		this.altTextFld.on('valid', this.onPropertyChange, this);
		this.constrainProportionsCheckbox.on('check', this.onConstrainProportionsChange, this);
		this.captionOptionsFld.on('select', this.onCaptionOptionSelect, this);
	},

	/**
	 * Clean up events
	 */
	onDestroy: function () {
		this.model.un('heightchange', this.afterHeightChange, this);
		this.model.un('alignchange', this.afterAlignChange, this);

		this.widthFld.un('valid', this.onSizeChange, this);
		this.widthFld.un('blur', this.onSizeBlur, this);
		this.heightFld.un('valid', this.onSizeChange, this);
		this.heightFld.un('blur', this.onSizeBlur, this);
		this.resetButton.on('click', this.onSizeReset, this);
		this.uploadFld.un('fileselected', this.onFileSelect, this);
		this.alignCtl.un('change', this.onPropertyChange, this);
		this.altTextFld.un('valid', this.onPropertyChange, this);
		this.constrainProportionsCheckbox.un('check', this.onConstrainProportionsChange, this);
		this.captionOptionsFld.un('select', this.onCaptionOptionSelect, this);

		this.model.selectInEditor();
		Sb.editor.ImageEditDialog.superclass.onDestroy.call(this);
	},

	/**
	 * Fill dialog from model
	 */
	fillDialogFromModel: function () {
		var d = this.model.data;
		this.previewLbl.setText(this.editDialogPreview + ': ' + d.sourceFileName);
		this.previewLbl.el.ellipsize();
		this.updatePreview(d.sourceUrl);
		this.widthFld.setRawValue(d.width);
		this.heightFld.setRawValue(d.height);
		this.alignCtl.setValue(d.alignTo);
		this.altTextFld.setRawValue(d.alternativeText);
		this.constrainProportionsCheckbox.setRawValue(d.isProportional);
		this.captionOptionsFld.setValue(d.caption);
	},

	onCaptionOptionSelect: function (field, record) {
		this.model.set('caption', record.data.value);
	},

	onSizeChange: function (field) {
		var secondDimension = (field.name == 'width') ? 'height': 'width';
		this.model.set(field.name, this[field.name + 'Fld'].getValue());
		this.model.updateDimension(secondDimension);
		this[secondDimension + 'Fld'].setRawValue(this.model.get(secondDimension));
	},

	onSizeBlur: function (field) {
		if (!field.isValid()) {
			var secondDimension = (field.name == 'width') ? 'height' : 'width',
				size = this.model.getProportionalSize(
					secondDimension,
					this[secondDimension + 'Fld'].getValue()
				);

			field.setValue(size);
		}
	},

	onSizeReset: function () {
		var originalSize = this.model.get('originalSize');

		this.model.unserialize(originalSize);
		this.widthFld.setRawValue(originalSize.width);
		this.heightFld.setRawValue(originalSize.height);
	},

	onConstrainProportionsChange: function (checkbox, isProportional) {
		this.model.set('isProportional', isProportional);
		if (isProportional) {
			this.model.updateDimension('height');
			this.heightFld.setRawValue(this.model.get('height'));
		}
	},

	/**
	 * Updates simple properties
	 */
	onPropertyChange: function () {
		this.model.unserialize({
			alignTo				: (this.alignCtl.getValue()) ? this.alignCtl.getValue().inputValue : 'none',
			alternativeText		: this.altTextFld.getValue()
		});
	},

	/**
	 * Sets focus after align has changed
	 */
	afterAlignChange: function () {
		this.focus();
	},

	/**
	 * Cancel button handler
	 */
	cancelHandler: function () {
		if (this.isUploaded) {
			this.model.removeFromEditor();
			this.afterHeightChange();
		} else {
			this.model.restoreFromMemento(this.memento);
		}
		this.destroy();
		tinyMCE.activeEditor.nodeChanged();
	},

	/**
	 * Fires when user selects image in file upload control
	 */
	onFileSelect: function (field) {
		var form = field.ownerCt.getForm();
		if (!form.isValid()) {
			field.markInvalid(this.editDialogChangeWarningInvalidType);
			return;
		}

		form.submit({
			url: this.uploadUrl,
			waitMsg: this.uploadingImage,
			scope: this,
			success: function (fp, action) {
				this.afterUploadFile(action.result);
				field.resetOnUpload();
			},
			failure: function (fp, action) {
				field.markInvalid(Sb.Locale.get(action.result.message));
			}
		});
	},

	afterUploadFile: function (result) {
		var root = Sb.Editor.model.getUrl();
		this.model.unserialize({
			sourceUrl: root + '/' + Sb.Config.path.attachmentsFolder + '/' + Sb.Config.path.imageFolder + '/' +
				encodeURIComponent(result.sourceFileName),
			isProportional: true,
			width: result.width,
			height: result.height,
			alignTo: Sb.Editor.model.isRtl() ? 'right' : 'left',
			originalSize: {
				width: result.width,
				height: result.height
			}
		});
		this.fillDialogFromModel();
		if (this.hidden) {
			this.show();
		}
	},

	okHandler: function () {
		if (this.altTextFld.validate()) {
			Sb.editor.ImageEditDialog.superclass.okHandler.call(this);
		}
	}
});