/**
 * Edit link dialog for text widget
 *
 * @class Sb.editor.LinkEditDialog
 * @extends Sb.Window
 */
Ext.define('Sb.editor.LinkEditDialog', {
	extend: 'Sb.Window',

	title		: Sb.Locale.get('linkEditDialogTitle'),
	layout		: 'fit',
	width		: 520,
	resizable	: false,
	modal		: true,

	initComponent: function () {
		this.pageRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('linkEditDialogPageField'),
			checked: true,
			name: 'linkType'
		});
		this.pageField = new Sb.form.PageComboBox({
			cls: 'sb-form-item-ident',
			emptyText: Sb.Locale.get('linkEditDialogPageFieldEmptyText'),
			listeners: {
				scope: this,
				focus: function () {
					this.pageRadio.setValue(true);
				}
			}
		});
		this.hrefRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('linkEditDialogHrefField'),
			name: 'linkType'
		});
		this.hrefField = new Ext.form.TextField({
			cls: 'sb-form-item-ident',
			listeners: {
				scope: this,
				focus: function () {
					this.hrefRadio.setValue(true);
				}
			}
		});
		this.documentRadio = new Ext.form.Radio({
			boxLabel: Sb.Locale.get('linkEditDialogDocumentField'),
			name: 'linkType'
		});
		this.documentField = new Sb.form.FileComboBox({
			cls: 'sb-form-item-ident',
			emptyText: Sb.Locale.get('linkEditDialogDocumentFieldEmptyText'),
			listeners: {
				scope: this,
				focus: function () {
					this.documentRadio.setValue(true);
				}
			}
		});

		Ext.apply(this, {
			items: [{
				ref: 'form',
				layout: 'form',
				autoHeight: true,
				border: false,
				labelWidth: 100,
				defaults: {
					anchor: '100%'
				},
				items: [{
					fieldLabel: Sb.Locale.get('linkEditDialogLinkField'),
					xtype: 'radiogroup',
					columns: 1,
					items: [
						this.pageRadio,
						this.pageField,
						this.hrefRadio,
						this.hrefField,
						{
							xtype: 'box',
							cls: 'sb-editor-box-description',
							html: Sb.Locale.get('linkEditDialogHrefHint')
						},
						this.documentRadio,
						this.documentField
					]
				}, {
					ref: '../targetField',
					fieldLabel: Sb.Locale.get('linkEditDialogTargetField'),
					xtype: 'combo',
					typeAhead: true,
					triggerAction: 'all',
					lazyRender: true,
					mode: 'local',
					value: '',
					store: new Ext.data.ArrayStore({
						fields: ['value', 'title'],
						data: [
							['', Sb.Locale.get('linkEditDialogTargetSelf')],
							['_blank', Sb.Locale.get('linkEditDialogTargetBlank')]
						]
					}),
					valueField: 'value',
					displayField: 'title'
				}, {
					ref: '../titleField',
					fieldLabel: Sb.Locale.get('linkEditDialogTitleField'),
					xtype: 'textfield'
				}, {
					xtype: 'box',
					fieldLabel: ' ',
					border: false,
					cls: 'sb-editor-box-description',
					html: Sb.Locale.get('linkEditDialogTitleHint')
				}]
			}],
			buttons: [{
				text: Sb.Locale.get('ok'),
				handler: this.onUpdate,
				scope: this
			}, {
				text: Sb.Locale.get('cancelButtonTitle'),
				handler: function () {
					this.close();
				},
				scope: this
			}]
		});
		Sb.editor.LinkEditDialog.superclass.initComponent.call(this);
	},

	show: function (editor) {
		this.editor = editor;
		this.linkEl = Sb.utils.Tinymce.getLinkNodeFromSelection(editor);

		Sb.editor.LinkEditDialog.superclass.show.call(this);

		if (this.linkEl && this.linkEl.nodeName == "A") {
			var target = editor.dom.getAttrib(this.linkEl, 'target');
			if (target != '_blank') {
				target = '';
			}
			this.setHref(editor.dom.getAttrib(this.linkEl, 'href'));
			this.targetField.setValue(target);
			this.titleField.setValue(editor.dom.getAttrib(this.linkEl, 'title'));
		}

	},

	setHref: function (href) {
		var isPage = href.match(/^page:\/\/([^\/]*)\/?$/),
			documentBaseUrl = Sb.Editor.model.get('publishWebSiteUrl') +
				(Sb.Config.isTrialMode ? 'attachments/documents' : 'data/documents');

		if (isPage) {
			var fullName = isPage[1],
				urlParts = fullName.split('#'),
				pageName = urlParts[0],
				anchorName = urlParts[1],
				pageHasAnchor = anchorName ? Sb.Editor.siteView.currentPage.hasAnchor(anchorName) : false;

			this.pageRadio.setValue(true);
			this.pageField.setValue(pageHasAnchor ? fullName : pageName);
			return;
		}

		if (href.indexOf(documentBaseUrl) === 0) {
			this.documentRadio.setValue(true);
			this.documentField.setValue(href);
			return;
		}

		this.hrefRadio.setValue(true);
		this.hrefField.setValue(href);
	},

	getHref: function () {
		var identity;

		if (this.pageRadio.getValue()) {
			identity = this.pageField.getValue();
			return identity ? 'page://' + identity : '';
		}

		if (this.documentRadio.getValue()) {
			identity = this.documentField.getValue();
			return identity;
		}

		return this.hrefField.getValue();
	},

	onUpdate: function () {
		var bookmark = Ext.isIE ? this.editor.lastSelectionBookmark : null;

		if (this.getHref() === '') {
			Sb.utils.Tinymce.removeNode(this.editor, this.linkEl, bookmark);
			this.close();
			return;
		}

		this.editor.execCommand("mceBeginUndoLevel");

		if (!this.linkEl) {
			this.linkEl = Sb.utils.Tinymce.createLinkNode(this.editor, bookmark);
		}

		this.editor.dom.setAttrib(this.linkEl, 'href', this.getHref());
		this.editor.dom.setAttrib(this.linkEl, 'target', this.targetField.getValue());
		this.editor.dom.setAttrib(this.linkEl, 'title', this.titleField.getValue());

		Sb.utils.Tinymce.selectNode(this.editor, this.linkEl);

		this.editor.execCommand("mceEndUndoLevel");
		this.close();
	},

	close: function () {
		Sb.editor.LinkEditDialog.superclass.close.call(this);
		this.editor.focus();
	}
});