/**
 * @class Sb.editor.Snapshot
 * @singleton
 */
Ext.define('Sb.editor.Snapshot', {
	singleton: true,

	snapshotStore: new Ext.data.ArrayStore({
		fields: ['title', {name: 'date', type: 'date', dateFormat: 'U'}, 'type']
	}),

	getSnapshotList: function (callback) {
		if (Sb.Editor.isDemoMode()) {
			return;
		}
		Sb.Loading.updateMessage(Sb.Locale.get('gettingSiteSnapshotList'));
		Sb.Loading.show();
		Sb.SiteService.getSiteSnapshotList(function (result, response) {
			if (response.result) {
				var i, data = [];
				for (i = 0; i < Sb.Config.snapshotSlots; i++) {
					data[i] = [Sb.Locale.get('emptySlot'), null, 'user'];
				}
				data[Sb.Config.snapshotSlots] = [Sb.Locale.get('quickSnapshotEmptyName'), null, 'quick'];
				Ext.each(response.result, function (snapshot) {
					var slotIndex = snapshot.slot != 'quick' ? snapshot.slot : Sb.Config.snapshotSlots;
					data[slotIndex] = [snapshot.title, snapshot.date, snapshot.slot != 'quick' ? 'user' : 'quick'];
				});
				this.snapshotStore.loadData(data);
			}
			Sb.Loading.hide();
			Ext.callback(callback);
		}, this);
	},

	generateSnapshotButton: function (type) {
		var text, tooltip, handler, isTitleEditable,
			isDisabled = Sb.Editor.isDemoMode();

		if (type == 'save') {
			text			= Sb.Locale.get('createSiteSnapshot');
			tooltip			= isDisabled ? Sb.Locale.get('createSiteSnapshotDisabled') : Sb.Locale.get('createSiteSnapshotDescription');
			handler			= this.onSaveSnapshotButton;
			isTitleEditable	= true;
		} else if (type == 'load') {
			text			= Sb.Locale.get('restoreSiteSnapshot');
			tooltip			= isDisabled ? Sb.Locale.get('restoreSiteSnapshotDisabled') : Sb.Locale.get('restoreSiteSnapshotDescription');
			handler			= this.onLoadSnapshotButton;
			isTitleEditable	= false;
		}

		this.button = new Ext.SplitButton({
			id			: type + 'SnapshotButton',
			text		: text,
			tooltip		: tooltip,
			disabled	: isDisabled,
			menu		: new Sb.editor.SnapshotMenu({
				type			: type,
				store			: this.snapshotStore,
				isTitleEditable	: isTitleEditable,
				isApplyOnEnter	: true,
				height			: (Sb.Config.snapshotSlots + 1) * 26 + 75
			}),
			cls: 'sb-editor-panel-split-button',
			scope		: this,
			handler		: handler.createDelegate(this),
			arrowHandler: function (button) {
				if (this.snapshotStore.getCount() === 0) {
					this.getSnapshotList(function () {
						button.showMenu();
					});
				}
			},
			listeners: {
				scope: this,
				afterrender: function (splitButton) {
					splitButton.el.child('button').hover(
						function () { splitButton.addClass('button-hover'); },
						function () { splitButton.removeClass('button-hover'); }
					);
				}
			}
		});
		this.button.menu.on('saveslot',	this.saveSnapshot, this);
		this.button.menu.on('loadslot',	this.loadSnapshot, this);
		this.button.menu.on('uploadslot',	this.uploadSnapshot, this);
		this.button.menu.on('downloadslot',	this.downloadSnapshot, this);
		this.button.menu.on('deleteslot',	this.deleteSnapshot, this);

		this.button.menu.on('beforeshow', function (menu) {
			if (this.snapshotStore.getCount() === 0) {
				return false;
			}
		}, this);

		this.button.menu.on('show', function (menu) {
			var recordToSelect = null, grid = menu.grid;
			if (type == 'save') {
				this.snapshotStore.each(function (record, index) {
					if (menu.isEmptySlot(index)) {
						recordToSelect = record;
						return false;
					}
				});
				if (null == recordToSelect) {
					this.snapshotStore.each(function (record, index) {
						if (null == recordToSelect || record.get('date') < recordToSelect.get('date')) {
							recordToSelect = record;
						}
					});
				}
				var rowIndex = this.snapshotStore.indexOf(recordToSelect);
				grid.getSelectionModel().selectRow(rowIndex);
				grid.startEditing.defer(200, grid, [rowIndex, 0]);
			} else {
				this.snapshotStore.each(function (record, index) {
					if (menu.isEmptySlot(index)) {
						return;
					}
					if (null == recordToSelect || record.get('date') > recordToSelect.get('date')) {
						recordToSelect = record;
					}
				});
				grid.getSelectionModel().selectRow(this.snapshotStore.indexOf(recordToSelect));
				grid.view.focusEl.focus.defer(50, grid.view.focusEl);
			}
		}, this);

		return this.button;
	},

	onSaveSnapshotButton: function () {
		if (!this.snapshotStore.getCount()) {
			this.getSnapshotList(this.onSaveSnapshotButton.createDelegate(this));
			return;
		}

		var snapshot = this.getSnapshotBySlot('quick');
		snapshot.set('title', Sb.Locale.get('quickSnapshotName'));
		snapshot.commit();
		this.saveSnapshot('quick');
	},

	onLoadSnapshotButton: function () {
		if (!this.snapshotStore.getCount()) {
			this.getSnapshotList(this.onLoadSnapshotButton.createDelegate(this));
			return;
		}

		var slot = this.getFreshSnapshotSlot();
		this.button.menu.onLoadSlot(slot == 'quick' ? Sb.Config.snapshotSlots : slot);
	},

	saveSnapshot: function (slot) {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.saveSnapshot.createDelegate(this, [slot]));
			return;
		}

		var snapshot = this.getSnapshotBySlot(slot),
			title = snapshot.get('title');

		if (title == Sb.Locale.get('quickSnapshotEmptyName')) {
			title = Sb.Locale.get('quickSnapshotName');
		}
		if (title == Sb.Locale.get('emptySlot')) {
			title = Sb.Locale.get('userSnapshotName');
		}

		Sb.Loading.updateMessage(Sb.Locale.get('creatingSiteSnapshot'));
		Sb.Loading.show();
		Sb.Editor.Ui.mainWindow.statusBar.clear();
		Sb.SiteService.createSiteSnapshot(slot, title, function (result, response) {
			if (response.result.success) {
				if (snapshot) {
					snapshot.set('title', title);
					snapshot.set('date', Date.parseDate(result.data.date, 'U'));
					snapshot.commit();
				}
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('siteSnapshotCreated')
				);
			}
			this.realignMenu(Ext.getCmp('saveSnapshotButton'));
			Sb.Loading.hide();
		}.createDelegate(this));
	},

	/**
	 * Load and apply snapshot data from slot
	 *
	 * @param {Number} slot Number of snapshot slot
	 * @param {Boolean} [allowRestrictedContent=false] allowRestrictedContent Restoring with restricted content
	 */
	loadSnapshot: function (slot, allowRestrictedContent) {
		allowRestrictedContent = !!(allowRestrictedContent);
		Sb.Editor.Ui.mainWindow.statusBar.clear();
		Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
		Sb.Loading.updateMessage(Sb.Locale.get('restoringSiteFromSnapshot'));
		Sb.Loading.show();
		Sb.SiteService.restoreSiteSnapshot(slot, allowRestrictedContent, function (result, response) {
			var buttonElement = Ext.get(this.button.menu.type + 'SnapshotButton');
			if (!response.result.success) {
				if (result.isContainRestrictedContent) {
					Sb.Loading.hide();
					Ext.Msg.show({
						width: 400,
						closable : false,
						title: Ext.util.Format.htmlEncode(Sb.Locale.get('shapshotRestrictedContentWarningTitle')),
						msg: Ext.util.Format.htmlEncode(Sb.Locale.get('shapshotRestrictedContentWarningMessage')),
						buttons: Ext.Msg.YESNO,
						scope: this,
						fn: function (btn) {
							if (btn == 'yes') {
								this.loadSnapshot(slot, true);
							}
						}
					});
					return;
				}

				var msg = '';
				if (Ext.isArray(response.result.message)) {
					msg = Sb.Locale.get(response.result.message[0], Sb.Config.general.product, response.result.message[1]);
				} else {
					msg = Sb.Locale.get(response.result.message);
				}
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(msg);
				this.button.menu.showAt(buttonElement.getAlignToXY(buttonElement));
				Sb.Loading.hide();
			} else {
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('siteSnapshotApplied')
				);
				this.button.menu.showAt(buttonElement.getAlignToXY(buttonElement));
				if (response.result.localeChanged) {
					window.location.reload();
				} else {
					Sb.Editor.model.load();
				}
			}
		}.createDelegate(this));
	},

	uploadSnapshot: function (form, slot, successCallback, failureCallback) {
		form.submit({
			params: {slot: slot},
			url: Sb.Config.baseUrl + '/Upload/Snapshot/',
			waitMsg: Sb.Locale.get('uploadingSnapshot'),
			scope: this,
			success: function (fp, action) {
				var snapshot = this.getSnapshotBySlot(slot);
					snapshot.set('title', action.result.title);
					snapshot.set('date', Date.parseDate(action.result.date, 'U'));
					snapshot.commit();
				Ext.callback(successCallback);
			},
			failure: function (fp, action) {
				Ext.callback(failureCallback, Sb.Locale.get(
					action.result && action.result.message ? action.result.message : 'messageNotSnapshotFile'
				));
			}
		});
	},

	downloadSnapshot: function (id) {
		window.location = Sb.Config.baseUrl + '/Download/Snapshot/?slot=' + id + '&X-CSRF-Token=' + Sb.Config.csrfToken;
	},

	deleteSnapshot: function (slot) {
		Sb.Loading.updateMessage(Sb.Locale.get('deletingSiteSnapshot'));
		Sb.Loading.show();
		Sb.SiteService.deleteSiteSnapshot(slot, function (result, response) {
			if (!response.result.success) {
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
					Sb.Locale.get(response.result.message)
				);
			} else {
				var title = slot == 'quick' ? 'quickSnapshotEmptyName' : 'emptySlot',
					snapshot = this.getSnapshotBySlot(slot);

				snapshot.set('title', Sb.Locale.get(title));
				snapshot.set('date', null);
				snapshot.commit();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					Sb.Locale.get('siteSnapshotDeleted')
				);
			}
			Sb.Loading.hide();
		}.createDelegate(this));
	},

	getFreshSnapshotSlot: function () {
		var slotDate,
			oldestDate = null,
			slot = null;

		this.snapshotStore.each(function (snapshot, position) {
			slotDate = snapshot.get('date');
			if (slotDate == null) {
				return;
			}
			if (oldestDate === null || slotDate > oldestDate) {
				oldestDate = slotDate;
				slot = this.button.menu.getSlotByIndex(position);
			}
		}, this);
		return slot;
	},

	getSnapshotBySlot: function (slot) {
		var index = slot == 'quick' ? Sb.Config.snapshotSlots : slot;
		return this.snapshotStore.getAt(index);
	},

	realignMenu: function (button) {
		if (!button.hasVisibleMenu()) {
			return;
		}
		var menu = button.menu;
		menu.el.setXY(menu.el.getAlignToXY(button.el, menu.defaultAlign, menu.defaultOffsets));
	}
});
