/**
 * Base class for all component edit panels
 *
 * @class Sb.editor.component.BaseEditPanel
 * @extends Ext.Panel
 */
Ext.define('Sb.editor.component.BaseEditPanel', {
	extend: 'Ext.Panel',

	autoRender: true,
	baseCls: 'sb-widget-edit-panel',
	frame: false,

	/**
	 * Try to show editPanel above (t) or under (b) widget
	 *
	 * @type String 't' or 'b'
	 * @property defaultPosition
	 */
	defaultPosition: null,

	/**
	 * @cfg {null|Array} offsets
	 * Array of coordinates for edit panel position offsets relative to widget.
	 */
	offsets: null,

	afterRender: function () {
		var editPanel = this;

		Sb.editor.component.BaseEditPanel.superclass.afterRender.call(this);

		if (this.component.movable) {
			this.el.addClass('movable');

			this.moveControl = this.el.createChild({
				cls: 'sb-widget-edit-panel-move',
				html: '&nbsp;'
			}, this.el.first());

			this.dd = new Ext.dd.DD(this, null, {
				moveOnly: true
			});
			this.dd.editPanel = this;
			this.dd.setHandleElId(this.moveControl);
			this.dd.startDrag = function () {
				editPanel.toFront();
				this.constrainTo(Ext.getBody());
				this.iframeFix = Ext.DomHelper.append(document.body, {tag: 'div', cls: 'draggable-iframe-overlay'});
			};
			this.dd.endDrag = function () {
				this.editPanel.offsets = this.editPanel.el.getOffsetsTo(this.editPanel.component.el);
				document.body.removeChild(this.iframeFix);
			};
		}

		this.el.on('click', this.toFront, this);
	},

	show: function (component) {
		this.component = component;
		Sb.editor.component.BaseEditPanel.superclass.show.call(this);
		this.reanchor();

		this.component.toFront();
		this.toFront();
	},

	hide: function () {
		Sb.editor.component.BaseEditPanel.superclass.hide.call(this);
		this.offsets = null;
	},

	reanchor: function (realign) {
		if (Sb.Editor.Ui.isDragActive()) {
			return;
		}

		if (this.component) {
			this.component.anchorControls();
		}

		var widgetX = this.component.el.getX(),
			widgetY = this.component.el.getY(),
			widgetBottom = this.component.el.getBottom(),
			widgetControlsY = this.component.controlsEl.getY(),
			widgetControlsOffset = 0,
			viewHeight = Ext.lib.Dom.getViewHeight(),
			bottomSpace = viewHeight - widgetBottom,
			x = widgetX,
			y,
			isComponentVisible = widgetY < viewHeight && widgetBottom > 0,
			editPanelHeight = this.el.getHeight();

		this.el.setVisible(isComponentVisible);
		if (!isComponentVisible) {
			return;
		}

		realign = Ext.isDefined(realign) ? realign : true;

		if (this.offsets) {
			x += this.offsets[0];
			y = widgetY + this.offsets[1];
			this.el.setXY([x, y]);

			if (this.el.getTop() < 0) {
				this.el.setY(0);
			}

			if (this.el.getBottom() > viewHeight) {
				this.el.setY(viewHeight - editPanelHeight);
			}

			return;
		}

		if (realign) {
			if (
				this.defaultPosition == 't' &&
				widgetY > editPanelHeight + widgetControlsOffset + 2
			) {
				this.position = 't';
			} else {
				this.position = widgetY > bottomSpace ? 't' : 'b';
			}
		}

		if (this.position == 't') {
			if (widgetControlsY < widgetY) {
				widgetControlsOffset = this.component.controlsEl.getHeight();
			}
			y = widgetY - editPanelHeight - widgetControlsOffset - 2;
		} else {
			if (widgetControlsY > widgetY) {
				widgetControlsOffset = this.component.controlsEl.getHeight() + 2;
			} else {
				widgetControlsOffset = 1;
			}
			y = widgetBottom + widgetControlsOffset;
		}

		if (y < 0) {
			y = 0;
		}

		if (y > viewHeight) {
			y = viewHeight - editPanelHeight;
		}

		this.el.setXY([x, y]);
	},

	/**
	 * Gets the x,y coordinates to align this element with another element.
	 *
	 * @param {Ext.Element/HTMLElement/String} element The element to align to.
	 * @param {String} [position="tl-bl?"] The position to align to.
	 * @param {Array} [offsets=[0,0]] Offset the positioning by [x, y]
	 * @return {Array} [x, y]
	 */
	getAlignToXY : function (element, position, offsets) {
		var me = this.el,
			controlsHeight = this.component.controlsEl.getHeight(),
			maxY = Ext.lib.Dom.getViewHeight() - me.getHeight() - this.component.el.getHeight() - controlsHeight - 30,
			xy = me.getAlignToXY(element, position, offsets);

		if (this.component.controlsEl.getY() > this.component.el.getY()) {
			offsets[1] += controlsHeight;
		}

		if (xy[1] < this.component.el.getY()) {
			xy[1] = Math.max(xy[1] - controlsHeight, 0);
		}

		if (xy[1] > maxY) {
			xy[1] = maxY;
		}

		return xy;
	},

	alignTo : function (element, position, offsets, animate) {
		var x = this.getAlignToXY(element, position, offsets),
			y = this.el.preanim && animate ? this.el.preanim(arguments, 3) : false;

		return this.el.setXY(x, y);
	},

	toFront: function () {
		var stylePanel = this.component.getStylePanel(),
			zIndex = parseInt(this.component.el.getStyle('z-index'), 10);

		this.el.setStyle('z-index', zIndex + 2);
		if (stylePanel && stylePanel.rendered) {
			stylePanel.el.setStyle('z-index', zIndex + 1);
		}
	}
});