/**
 * Class for image component edit panel
 *
 * @class Sb.editor.component.ImageEditPanel
 * @extends Sb.editor.component.BaseEditPanel
 */
Ext.define('Sb.editor.component.ImageEditPanel', {
	extend: 'Sb.editor.component.BaseEditPanel',

	cls: 'sb-component-edit-panel-image',
	autoHeight: true,
	showAlignment: false,
	uploadFieldLabel: Sb.Locale.get('componentImageUploadFieldTitle'),
	alignmentFieldLabel: Sb.Locale.get('componentImageAlignmentFieldTitle'),
	uploadAction: '',
	originalImageSize: {width: 0, height: 0},
	validExtensions: Sb.Config.imageValidExtensions,

	initComponent: function () {
		this.addEvents('imagechange', 'sizechange', 'alignmentchange', 'proportionschange');

		Ext.each(['width', 'height'], function (dimension) {
			this[dimension + 'Field'] = new Sb.form.SizeField({
				name: dimension,
				listeners: {
					scope: this,
					valid: this.onSizeChange,
					blur: this.onSizeBlur
				}
			});
		}, this);

		this.constrainProportionsCheckbox = new Ext.form.Checkbox({
			hideLabel: true,
			boxLabel: Sb.Locale.get('imageEditDialogSizeConstrainProportions'),
			listeners: {
				scope: this,
				check: this.onConstrainProportionsChange
			}
		});

		this.uploadField = new Ext.ux.form.FileUploadField({
			fieldLabel: this.uploadFieldLabel,
			name: 'uploadfile',
			msgTarget: 'under',
			width: 291,
			regex: new RegExp('\\.(' + this.validExtensions.join('|') + ')$', 'i'),
			regexText: Sb.Locale.get('imageUploadErrorExtensionInvalid'),
			buttonText: Sb.Locale.get('browseButtonText'),
			emptyText: Sb.Locale.get('clickToBrowseAndUploadFile'),
			listeners: {
				scope: this,
				fileselected: this.onImageSelected
			}
		});

		var alignmentButtonConfig = {
			toggleGroup: 'image-alignment',
			listeners: {
				scope: this,
				toggle: this.onAlignmentChange
			}
		};

		Ext.each(['left', 'center', 'right'], function (position) {
			this[position + 'AlignmentButton'] = new Ext.Button(
				Ext.apply({
					iconCls: 'x-edit-alignment-' + position,
					value: position
				}, alignmentButtonConfig)
			);
		}, this);

		var sizeFields = {
			layout: 'column',
			items: [this.widthField, {
				cls: 'x',
				html: 'x'
			}, this.heightField, {
				xtype: 'button',
				text: Sb.Locale.get('imageEditDialogSizeReset'),
				cls: 'sb-image-size-control-reset',
				handler: this.onSizeReset.createDelegate(this)
			}]
		};

		var sizeControl = {
			xtype: 'container',
			fieldLabel: Sb.Locale.get('imageEditDialogSizeInPixels'),
			cls: 'sb-image-size-control',
			items: [sizeFields, this.constrainProportionsCheckbox]
		};

		var formSecondPart;

		if (this.showAlignment) {
			var alignmentControl = {
				xtype: 'container',
				fieldLabel: this.alignmentFieldLabel,
				layout: 'column',
				itemCls: 'sb-edit-control',
				items: [this.leftAlignmentButton, this.centerAlignmentButton, this.rightAlignmentButton]
			};
			formSecondPart = {
				layout: 'column',
				defaults: {
					layout: 'form',
					labelAlign: 'top'
				},
				items: [{
					width: 185,
					items: sizeControl
				}, {
					width: 113,
					items: alignmentControl
				}]
			};
		} else {
			formSecondPart = {
				layout: 'form',
				labelAlign: 'top',
				items: sizeControl
			};
		}

		this.items = [{
			xtype: 'form',
			ref: 'formPanel',
			fileUpload: true,
			labelAlign: 'top',
			enableKeyEvents: true,
			items: this.uploadField
		}, formSecondPart];

		Sb.editor.component.ImageEditPanel.superclass.initComponent.apply(this, arguments);
	},

	updateFields: function (config) {
		if (config.isProportional !== undefined) {
			this.constrainProportionsCheckbox.setRawValue(config.isProportional);
		}

		if (config.alignment && this.showAlignment) {
			var alignment = config.alignment;
			if (Sb.Editor.model.isRtl()) {
				alignment = Sb.Utils.swapLeftAndRight(alignment);
			}
			this[alignment + 'AlignmentButton'].toggle(true);
		}

		if (config.imageSize) {
			this.widthField.setRawValue(config.imageSize.width);
			this.heightField.setRawValue(config.imageSize.height);
		}
	},

	show: function (component) {
		Sb.editor.component.ImageEditPanel.superclass.show.apply(this, arguments);
		this.uploadField.reset();
	},

	onImageSelected: function (field, value) {
		if ('' === value) {
			return;
		}

		this.formPanel.form.submit({
			url: this.uploadAction,
			waitMsg: Sb.Locale.get('uploadingImage'),
			scope: this,
			success: function (fp, action) {
				this.fireEvent('imagechange', this, action.result.fileName);
				field.resetOnUpload();
			},
			failure: function (fp, action) {
				if (action.result) {
					field.markInvalid(Sb.Locale.get(action.result.message));
				}
			}
		});
	},

	onAlignmentChange: function (btn, pressed) {
		if (pressed) {
			var value = Sb.Editor.model.isRtl() ? Sb.Utils.swapLeftAndRight(btn.value) : btn.value;
			this.alignment = value;
			this.fireEvent('alignmentchange', this, value);
		}
	},

	onSizeChange: function (field) {
		var newSize = {},
			secondDimension = this.getSecondDimension(field.name);

		if (this.isProportional()) {
			var secondSize = this.getProportionalSize(field.name, field.getValue());

			newSize[secondDimension] = secondSize;
			this[secondDimension + 'Field'].setRawValue(secondSize);
		} else {
			newSize[secondDimension] = this[secondDimension + 'Field'].getValue();
		}

		newSize[field.name] = field.getValue();
		this.fireEvent('sizechange', this, newSize);
	},

	onSizeBlur: function (field) {
		if (!field.isValid()) {
			var secondDimension = this.getSecondDimension(field.name),
				secondSize = this[secondDimension + 'Field'].getValue(),
				size = this.getProportionalSize(secondDimension, secondSize),
				newSize = {};

			field.setRawValue(size);
			field.clearInvalid();

			newSize[field.name] = size;
			newSize[secondDimension] = secondSize;

			this.fireEvent('sizechange', this, newSize);
		}
	},

	onSizeReset: function () {
		this.widthField.setRawValue(this.originalImageSize.width);
		this.heightField.setRawValue(this.originalImageSize.height);

		this.fireEvent('sizechange', this, this.originalImageSize);
	},

	onConstrainProportionsChange: function (checkbox) {
		var checked = checkbox.getValue();

		if (checked) {
			var width = this.widthField.getValue(),
				height = this.getProportionalSize('width', width);

			this.heightField.setRawValue(height);
			this.fireEvent('sizechange', this, {width: width, height: height});
		}

		this.fireEvent('proportionschange', this, checked);
	},

	isProportional: function () {
		return this.constrainProportionsCheckbox.getValue();
	},

	/**
	 * Calculate second proportional size of image by first size
	 * @param {String} dimension 'width' or 'height'
	 * @param {Number} value second size in px
	 * @return {Number}
	 */
	getProportionalSize: function (dimension, value) {
		var secondDimension = this.getSecondDimension(dimension);

		return (this.originalImageSize[dimension] > 0)
			? Math.round(value * this.originalImageSize[secondDimension] / this.originalImageSize[dimension])
			: 0;
	},

	/**
	 * Calculate second proportional size of image by first size
	 * @param {String} dimension 'width' or 'height'
	 * @return {String} second dimension 'height' or 'width'
	 */
	getSecondDimension: function (dimension) {
		return (dimension === 'width') ? 'height' : 'width';
	}
});