/**
 * @class Sb.editor.component.Text
 * @extends Sb.editor.component.Base
 * Class for text component
 * @constructor
 * @param {Object} config The config object
 * @xtype text_component
 */
Ext.define('Sb.editor.component.Text', {
	extend: 'Sb.editor.component.Base',
	xtype: 'text_component',

	removeConfirmation: false,

	/**
	 * @cfg {String} emptyText
	 * Fallback text when component's content is empty (defaults to <code>Sb.Locale.get('textComponentEmptyMessage')</code>).
	 */
	emptyText: Sb.Locale.get('textComponentEmptyMessage'),

	/**
	 * @cfg {String} defaultText
	 * Default text applied to empty component's content (defaults to <code></code>).
	 */
	defaultText: '',

	/**
	 * @cfg {Boolean} isMultiline
	 * True to allow multiline
	 */
	isMultiline: true,

	/**
	 * @cfg {Number} maxTextLength
	 * Maximum length of innerHTML text.
	 * If value equal -1 that unlimited length
	 */
	maxTextLength: -1,

	/**
	 * @cfg {Boolean} isShowInvalidMsg
	 * Show or not invalid message under text
	 */
	isShowInvalidMsg: true,

	initComponent: function () {
		Sb.editor.component.Text.superclass.initComponent.apply(this, arguments);
		this.addEvents('clickoutsideedit');
		this.removeTitle = Sb.Locale.get('hideWidgetTitle');
		this.data = {};
	},

	getEditPanel: function () {
		if (!this.hasEditPanel) {
			return null;
		}
		if (!Sb.editor.component.Text.editPanel) {
			Sb.editor.component.Text.editPanel = new Sb.editor.component.TextEditPanel();
		}
		return Sb.editor.component.Text.editPanel;
	},

	onStartEdit: function () {
		Sb.editor.component.Text.superclass.onStartEdit.apply(this);
		this.memento = Ext.apply({}, this.data);
		this.contentElement.set({contentEditable: 'true'});
		if (this.contentElement.dom.innerHTML == this.emptyText) {
			this.contentElement.update('');
		}
		this.contentFocus();
		this.contentElement.on('keyup', this.onKeyUp, this);
		this.contentElement.on('keydown', this.onKeyDown, this);
		this.contentElement.on('paste', this.onPaste, this);
	},

	onPaste: function (evt) {
		// :NOTE: smikhalyev 20110504: Emulation afterPaste event
		setTimeout(this.onAfterPaste.createDelegate(this), 0);
	},

	onAfterPaste: function () {
		var dom = this.contentElement.dom,
			newContent = dom.innerHTML;

		if (dom) {
			var badNodes = Ext.fly(dom).query('*'),
				isOriginalNode = '_original_contentElement_node_';
			// :INFO: austimenko@ 20110621: It's a workaround for FF copy/paste issue
			if (badNodes.length > 1 && dom.textContent) {
				newContent = dom.textContent;
				this.contentElement.addClass(isOriginalNode);
				Ext.each(badNodes, function (badNode, index) {
					if (Ext.fly(badNode).hasClass(isOriginalNode)) {
						return;
					}
					Ext.fly(badNode).remove();
				});
				this.contentElement.removeClass(isOriginalNode);
			}
		}

		if (this.isMultiline) {
			newContent = Ext.util.Format.nl2br(newContent);
			newContent = Ext.util.Format.stripTags(newContent, '<br>');
		} else {
			newContent = newContent.replace(/\r?\n/g, '');
			newContent = Ext.util.Format.stripTags(newContent);
		}

		this.contentElement.update(newContent);
		this.validate();
	},

	markInvalid: function (msg) {
		var invalidCls				= this.baseCls + '-textcomponent-invalid',
			invalidContentCls		= this.baseCls + '-textcomponent-content-invalid';

		this.el.addClass(invalidCls);
		this.contentElement.addClass(invalidContentCls);

		if (!this.isShowInvalidMsg) {
			return;
		}
		if (!this.invalidMsgEl) {
			this.invalidMsgEl = Ext.get(Ext.DomHelper.insertAfter(
				this.el, {cls: 'x-form-invalid-msg'}
			));
		}
		this.invalidMsgEl.update(msg);
	},

	clearInvalid: function () {
		var invalidCls			= this.baseCls + '-textcomponent-invalid',
		invalidContentCls		= this.baseCls + '-textcomponent-content-invalid';
		this.el.replaceClass(invalidCls, '');
		this.contentElement.replaceClass(invalidContentCls, '');
		if (this.invalidMsgEl) {
			this.invalidMsgEl.remove();
			delete(this.invalidMsgEl);
		}
	},

	validate: function () {
		var isValid = this.isValid();
		if (isValid) {
			this.clearInvalid();
		} else {
			this.markInvalid(Sb.Locale.get('widgetComponentTextMaxLength', this.maxTextLength));
		}
		return isValid;
	},

	isValid: function () {
		var content = this.contentElement.dom.innerHTML;
		return (this.maxTextLength < 0 || content.length <= this.maxTextLength);
	},

	completeEdit: function () {
		if (!this.validate()) {
			return;
		}
		Sb.editor.component.Text.superclass.completeEdit.apply(this, arguments);
	},

	onCompleteEdit: function () {
		this.setData({content: this.contentElement.dom.innerHTML});
		Sb.editor.component.Text.superclass.onCompleteEdit.apply(this, arguments);
	},

	onCancelEdit: function () {
		Sb.editor.component.Text.superclass.onCancelEdit.apply(this);
		this.setData(this.memento);
		this.clearInvalid();
	},

	onHideEdit: function () {
		this.contentElement.set({contentEditable: 'false'});
		Sb.editor.component.Text.superclass.onHideEdit.apply(this);
	},

	onKeyUp: function (e) {
		this.validate();
		e.stopPropagation();
		this.data.content = this.contentElement.dom.innerHTML;
		if (this.hasEditPanel && this.contentElement.dom.innerHTML !== '') {
			this.getEditPanel().reanchor();
		}
	},

	onDocumentClick: function (event) {
		if (false === this.fireEvent('clickoutsideedit', this, event)) {
			return;
		}

		Sb.editor.component.Text.superclass.onDocumentClick.call(this, event);
	},

	onKeyDown: function (e) {
		e.stopPropagation();
		if (!this.isMultiline && e.getKey() == e.ENTER) {
			e.preventDefault();
			this.completeEdit(true);
		}
		if (e.isSpecialKey()) {
			return;
		}
		var isLengthExceeded = (this.maxTextLength >= 0 && this.contentElement.dom.innerHTML.length >= this.maxTextLength);

		if (isLengthExceeded) {
			e.preventDefault();
		}
	},

	setContent: function (content) {
		var data		= this.data,
			stripTags	= Ext.util.Format.stripTags,
			trim		= function (text) {
				return text.replace(/^(\&nbsp;|\s)+||(\&nbsp;|\s)+$/, '');
			},
			hasVisibleSymbols	= function (text) {
				text = text.replace(/\&nbsp;/g, '');
				text = Ext.util.Format.stripTags(text);
				text = Ext.util.Format.trim(text);
				return (text !== '');
			},
			saveTags	= this.isMultiline ? '<br><br/>' : '';

		content	= content	.replace(/<\/(p|div)>/gi,		'')			// Clean up all closing tags
							.replace(/^<(p|div)[^>]*>/i,	'')			// Clean up 1st opening tag
							.replace(/<(p|div)[^>]*>/gi,	'<br/>');	// Replace remaining opening tags

		content = stripTags(content, saveTags);
		content	= content.replace(/(<br\/?>)*$/, '');					// Delete all <br> at the end

		content = (hasVisibleSymbols(content)) ? trim(content) : '';

		data.content = content;

		if (this.rendered) {
			var viewContent = (data.content !== '' && data.content != this.defaultText) ? data.content : this.emptyText;
			this.contentElement.update(viewContent);
			if (this.hasEditPanel && Sb.Editor.Ui.editComponent && Sb.Editor.Ui.editComponent == this) {
				this.getEditPanel().reanchor();
			}
		}
	},

	setData: function (newData) {
		Ext.apply(this.data, newData);
		var data = this.data;
		if ('content' in newData) {
			this.setContent(newData.content);
		}

		if (this.rendered && this.hasEditPanel) {
			var fontColor = Sb.Editor.model.design.getColor(data.fontColor),
				style = {
				'color': fontColor,
				'font-weight': (data.fontBold == 1) ? 'bold' : 'normal',
				'font-size': data.fontSize + 'px',
				'font-style': (data.fontItalic == 1) ? 'italic' : 'normal',
				'font-family': data.fontFamily,
				'text-decoration': (data.fontUnderline == 1) ? 'underline' : 'none'
			};
			this.contentElement.setStyle(style);

			this.contentElement.removeClass(['stroke-white', 'stroke-black']);
			if (data.stroke == '1') {
				var strokeColor = Ext.ux.util.Color.isLight(fontColor) ? 'black' : 'white';
				this.contentElement.addClass('stroke-' + strokeColor);
			}

			if (Sb.Editor.Ui.editComponent && Sb.Editor.Ui.editComponent == this) {
				this.getEditPanel().reanchor();
			}

			Sb.FontManager.loadFont(data.fontFamily);
		}
	},

	contentFocus: function () {
		this.contentElement.focus();

		if (Ext.isIE) {
			var content = this.contentElement.dom.innerHTML;
			var position = document.selection.createRange();
			position.moveStart("textedit", content.length);
			position.collapse(false);
			position.select();
		}
	},

	initDD: function () {
		this.dd = new Ext.dd.DD(this);
		this.dd.setOuterHandleElId(this.ddEl);
		this.dd.component = this;
	}
});