/**
 * @class Sb.menu.ColorMenu
 * @extends Ext.menu.Menu
 */
Ext.define('Sb.menu.ColorMenu', {
	extend: 'Ext.menu.Menu',

	/**
	 * @cfg {Boolean} allowBackground Is allow background image (gradient/custom/etc.)
	 */
	allowBackground: false,

	/**
	 * @cfg {Boolean} allowHeaders Is allow header library for selection background
	 */
	allowHeaders: false,

	/**
	 * @cfg {Boolean} allowOpacity Is allow opacity for solid color background and images
	 */
	allowOpacity: false,

	/**
	 * @cfg {Boolean} allowStroke True to allow select outline (defaults to false).
	 */
	allowStroke: false,

	/**
	 * @cfg {Boolean} allowInherit True to allow select inherit (Not set) color (defaults to false).
	 */
	allowInherit: false,

	/**
	 * @cfg {String} parentColor Real value for inherit (Not set) color (defaults to '#000000').
	 */
	parentColor: '#000000',

	/**
	 * @cfg {Boolean} allowColorPalette True to allow select colors from color scheme (defaults to false).
	 */
	allowColorPalette: false,

	constructor: function (config) {
		config = Ext.apply({
			allowBlank					: false, // Is allow transparent color and show "No color" button
			isShowColorField			: false,
			enableScrolling				: false,
			shadow						: false
		}, config);

		config.width = config.allowBackground ? 252 : 212;

		this.colorDropdowns = [];

		this.callParent([config]);
	},

	onRender: function () {
		this.keyNav = true;
		this.callParent(arguments);
		this.keyNav = false;
		this.el.on('mousedown', function () {
			if (this.activeChild) {
				this.activeChild.hide();
			}
		}, this);
	},

	initComponent: function () {
		var panelItems = [],
			colorTypeItems = [],
			colorPaletteColorsNames = [
				['color1-4', 'color1-3', 'color1', 'color1-1', 'color1-2'],
				['color2-4', 'color2-3', 'color2', 'color2-1', 'color2-2'],
				['color3-4', 'color3-3', 'color3', 'color3-1', 'color3-2'],
				['color4-4', 'color4-3', 'color4', 'color4-1', 'color4-2']
			],
			colorPaletteColors = [],
			gradients = [];

		if (this.allowBackground) {
			// Opacity.
			if (this.allowOpacity) {
				this.opacityField = new Ext.form.SliderField({
					fieldLabel: Sb.Locale.get('backgroundOpacity'),
					minValue: 0,
					maxValue: 100,
					tipText: function(thumb) {
						return thumb.value || '0';
					},
					itemCls: 'sb-color-dropdown-opacity',
					listeners: {
						scope: this,
						valid: function(slider) {
							var value = slider.getValue();

							if (value == this.backgroundValue.opacity) {
								return;
							}

							this.backgroundValue.opacity = value;
							this.fireEvent('backgroundchange', this, this.backgroundValue);
						}
					}
				});
				this.opacityPanel = new Ext.Container({
					layout: 'form',
					labelWidth: 75,
					labelAlign: 'right',
					items: this.opacityField
				});
				panelItems.push(this.opacityPanel);
			}

			this.backgroundType = new Ext.form.RadioGroup({
				hideLabel: true,
				cls: 'sb-radiogroup-inline sb-color-dropdown-background-type',
				defaults: {
					name: this.id + '-background-type'
				},
				items: [{
					boxLabel: Sb.Locale.get('backgroundSolidColor'),
					inputValue: 'none'
				}, {
					boxLabel: Sb.Locale.get('backgroundGradient'),
					inputValue: 'gradient'
				}, {
					boxLabel: Sb.Locale.get('backgroundPattern'),
					inputValue: 'pattern'
				}, {
					boxLabel: Sb.Locale.get('backgroundImage'),
					inputValue: 'image',
					hidden: !this.allowHeaders
				}],
				listeners: {
					scope: this,
					change: this.onBackgroundTypeChange
				}
			});
			panelItems.push(this.backgroundType);
		}

		if (this.allowStroke) {
			this.strokeField = new Ext.form.Checkbox({
				boxLabel: Sb.Locale.get('stroke'),
				hideLabel: true,
				itemCls: 'sb-color-dropdown-stroke'
			});
			panelItems.push(new Ext.Container({
				layout: 'form',
				items: this.strokeField
			}));
		}

		//no color button
		colorTypeItems.push({
			boxLabel: Sb.Locale.get('transparentColorDescription') +
				'<div class="sb-color-palette-item sb-color-dropdown-transparent-button"></div>',
			inputValue: 'transparent',
			itemCls: 'sb-color-dropdown-transparent'
		});

		if (this.allowInherit) {
			colorTypeItems.push({
				boxLabel: Sb.Locale.get('inheritColorDescription') +
					'<div class="sb-color-palette-item sb-color-dropdown-inherit-button"></div>',
				inputValue: 'inherit',
				itemCls: 'sb-color-dropdown-inherit'
			});
		}

		if (this.allowColorPalette) {
			Ext.each(colorPaletteColorsNames, function (colorPaletteColorsColumn, columnIndex) {
				colorPaletteColors[columnIndex] = [];
				Ext.each(colorPaletteColorsColumn, function (colorName, index) {
					colorPaletteColors[columnIndex][index] = colorName;
				});
			});

			this.colorPalette = new Sb.form.ColorPalette({
				colors: colorPaletteColors,
				listeners: {
					scope: this,
					select: function (palette, color) {
						this.setValue(color);
					}
				}
			});
			colorTypeItems.push({
				boxLabel: Sb.Locale.get('colorMenuColorScheme'),
				inputValue: 'palette'
			});
			colorTypeItems.push(this.colorPalette);
		}

		//picker
		this.picker = new Ext.ux.ColorPicker();
		this.picker.on('change', function (e, color) {
			this.setValue(color);
			Ext.each(this.colorDropdowns, function (field) {
				field.setValue(color);
			});
		}, this);

		//color field
		this.colorField = new Sb.form.ColorField({
			value			: this.value,
			currentPicker	: this.picker,
			selectedCls		: 'sb-colorfield-wrapper'
		});
		this.colorField.on('valid', function (field) {
			this.setValue(field.getValue());
		}, this);

		colorTypeItems.push({
			boxLabel: Sb.Locale.get('colorMenuCustomColor') +
				'<div class="sb-color-palette-item sb-color-dropdown-custom-button"></div>',
			inputValue: 'custom',
			itemCls: 'sb-color-dropdown-custom'
		});

		if (this.isShowColorField) {
			colorTypeItems.push(this.colorField);
		}
		colorTypeItems.push(this.picker);

		this.colorType = new Ext.form.RadioGroup({
			columns: 1,
			defaults: {
				name: this.id + '-color-type'
			},
			items: colorTypeItems,
			listeners: {
				scope: this,
				afterrender: this.updateColorButtons,
				change: this.onColorTypeChange
			}
		});
		panelItems.push(this.colorType);

		if (this.allowBackground) {
			this.backgroundColorWrapper = new Ext.Container({
				layout: 'form',
				labelWidth: 50,
				items: this.createColorDropdown({
					fieldLabel: Sb.Locale.get('backgroundColor')
				})
			});
			panelItems.push(this.backgroundColorWrapper);

			this.backgroundGradient = this.createGradientField();
			panelItems.push(this.backgroundGradient);

			this.backgroundPattern = this.createPatternField();
			panelItems.push(this.backgroundPattern);

			this.backgroundImage = this.createImageField();
			panelItems.push(this.backgroundImage);

			this.backgroundStretchType = new Ext.ux.IconCombo({
				editable: false,
				mode: 'local',
				store: new Ext.data.ArrayStore({
					fields: ['value', 'name', 'iconCls'],
					data: [
						['tile', Sb.Locale.get('tileImage'), 'sb-background-position-icon-tile '],
						['actual size', Sb.Locale.get('actualSize'), 'sb-background-position-icon-actual-size'],
						['crop', Sb.Locale.get('cropToFill'), 'sb-background-position-icon-crop-to-fill'],
						['resize', Sb.Locale.get('resizeToFit'), 'sb-background-position-icon-resize-to-fit']
					]
				}),
				displayField: 'name',
				valueField: 'value',
				iconClsField: 'iconCls',
				value: 'tile',
				itemCls: 'sb-editor-style-colors-background-position',
				listeners: {
					scope: this,
					select: function (combo, record) {
						this.backgroundValue.stretch = record.data.value;
						this.fireEvent('backgroundchange', this, this.backgroundValue);
					}
				}
			});

			this.backgroundPosition = new Ext.ux.IconCombo({
				autoSelect: true,
				editable: false,
				lazyInit: false,
				mode: 'local',
				triggerAction: 'all',
				store: new Ext.data.ArrayStore({
					fields: ['value', 'name', 'iconCls'],
					data: [
						['top left', Sb.Locale.get('topLeft'), 'sb-background-position-icon-tl'],
						['top center', Sb.Locale.get('topCenter'), 'sb-background-position-icon-tc'],
						['top right', Sb.Locale.get('topRight'), 'sb-background-position-icon-tr'],
						['center left', Sb.Locale.get('centerLeft'), 'sb-background-position-icon-cl'],
						['center center', Sb.Locale.get('centerCenter'), 'sb-background-position-icon-cc'],
						['center right', Sb.Locale.get('centerRight'), 'sb-background-position-icon-cr'],
						['bottom left', Sb.Locale.get('bottomLeft'), 'sb-background-position-icon-bl'],
						['bottom center', Sb.Locale.get('bottomCenter'), 'sb-background-position-icon-bc'],
						['bottom right', Sb.Locale.get('bottomRight'), 'sb-background-position-icon-br']
					]
				}),
				displayField: 'name',
				valueField: 'value',
				iconClsField: 'iconCls',
				value: 'top left',
				itemCls: 'sb-editor-style-colors-background-position',
				listeners: {
					scope: this,
					select: function (combo, record) {
						this.backgroundValue.position = record.data.value;
						this.fireEvent('backgroundchange', this, this.backgroundValue);
					}
				}
			});

			panelItems.push(new Ext.Container({
				cls: 'sb-color-dropdown-background-props',
				items: [
					this.backgroundStretchType,
					this.backgroundPosition
				]
			}));

			Sb.DesignTemplateService.getDecorationVariants('generic', 'background', [], function (result) {
				var backgrounds = [];

				Ext.iterate(result, function (item) {
					backgrounds.push([item, false]);
				});
				this.backgroundPattern.store.loadData(backgrounds, true);
				this.backgroundPattern.store.multiSort([this.backgroundPattern.store.sortInfo]);
			}, this);
			this.backgroundPattern.store.loadCustomBackgrounds();

			Ext.iterate(Sb.Editor.model.data.template.decorations.gradients, function (gradient, params) {
				gradients.push([
					gradient,
					params.direction,
					params.startColor,
					params.transparency / 100
				]);
			});
			this.backgroundGradient.store.loadData(gradients);
		}

		Ext.apply(this, {
			cls: 'sb-color-dropdown ' + this.cls,
			items: [{
				xtype: 'panel',
				items: {
					xtype: 'container',
					height: this.allowBackground ? 430 : 'auto',
					items: panelItems
				},
				buttonAlign: 'center',
				buttons: [{
					text: Sb.Locale.get('ok'),
					scope: this,
					handler: this.hide
				}, {
					text: Sb.Locale.get('cancelButtonTitle'),
					scope: this,
					handler: function () {
						this.isCancelled = true;
						if (this.allowBackground) {
							this.fireEvent('backgroundchange', this, this.initialBackgroundValue);
						}
						this.setValue(this.initialValue);
						if (this.allowOpacity) {
							this.opacityField.setRawValue(this.initialBackgroundValue.opacity);
						}
						if (this.allowStroke) {
							this.strokeField.setValue(this.initialStrokeValue);
						}
						this.hide();
					}
				}]
			}]
		});
		this.callParent();
	},

	createColorDropdown: function (config) {
		var field = new Sb.form.ColorDropDown(Ext.apply({
			parentMenu: this,
			width: 112,
			allowBlank: this.allowBlank,
			allowColorPalette: this.allowColorPalette,
			listeners: {
				scope: this,
				change: function (dropdown, color) {
					if (!dropdown.isValid()) {
						return;
					}
					this.setValue(color);
					Ext.each(this.colorDropdowns, function (field) {
						if (field.getId() !== dropdown.getId()) {
							field.setValue(color);
						}
					});
				}
			}
		}, config));
		this.colorDropdowns.push(field);
		return field;
	},

	hide: function () {
		this.callParent(arguments);
		if (this.parentMenu && this.parentMenu.activeChild === this) {
			this.parentMenu.activeChild = null;
		}
	},

	onDeleteBackground: function (selectbox, record) {
		var index = selectbox.store.indexOf(record);
		Sb.Editor.model.design.deleteBackground(record.get('name'));
		selectbox.store.remove(record);

		if (this.backgroundValue.pattern === record.get('name')) {
			if (index === selectbox.store.getCount()) {
				index--;
			}
			this.backgroundValue.pattern = selectbox.store.getAt(index).get('name');
			this.backgroundPattern.setValue(this.backgroundValue.pattern);
			this.fireEvent('backgroundchange', this, this.backgroundValue);
		}
	},

	createGradientField: function () {
		var colorDropdown = this.createColorDropdown({
			getMenuParent: function () {
				return this.el.up('.sb-selectbox-list') || document.body;
			}
		});
		return new Sb.form.SelectBox({
			fieldLabel: Sb.Locale.get('backgroundGradient'),
			name: 'backgroundGradient',
			width: 238,
			mode: 'local',
			store: new Ext.data.ArrayStore({
				fields: ['name', 'direction', 'startColor', 'opacity']
			}),
			lazyInit: false,
			valueField: 'name',
			cls: 'sb-gradient-images',
			triggerClass: 'sb-background-trigger',
			listWidth: 343,
			itemTpl: '<div style="opacity: {opacity}; background-image: linear-gradient(' +
					'to {direction},{startColor},transparent);"></div>',
			itemClass: 'sb-gradient-images-list-item',
			doFilter: function () {
				var numbers = '';
				var color = Ext.getCmp('invert-color-black').checked ? 'black' : 'white';
				if (Ext.getCmp('filter-vertical').checked) {
					numbers += '|2|3|7|10|11|14|15|18|19|22|23';
				}
				if (Ext.getCmp('filter-horizontal').checked) {
					numbers += '|4|6|8|9|12|13|16|17|20|21|24|25';
				}
				this.store.filterBy(function (record) {
					var name = record.get('name');
					if (name === '') {
						return true;
					}
					var filterRe = new RegExp('background(' + numbers + ')-' + color, 'i');
					return filterRe.test(name);
				});

				var oldValue = this.getValue(),
					newValue = oldValue.replace(/(black|white)$/, color);
				if (newValue != oldValue) {
					var r = this.findRecord(this.valueField || this.displayField, newValue);
					if (r) {
						this.select(this.store.indexOf(r), true);
						this.setValue(newValue);
						this.fireEvent('select', this, r, this.selectedIndex);
					}
				}
			},
			lastQuery: '',
			listeners: {
				scope: this,
				select: function (selectbox) {
					this.backgroundValue.gradient = selectbox.getValue();
					this.fireEvent('backgroundchange', this, this.backgroundValue);
				},
				afterrender: function (selectbox) {
					selectbox.header = selectbox.list.createChild(
						{cls: 'x-combo-list-hd'},
						selectbox.innerList
					);

					selectbox.settingsForm = new Ext.Container({
						renderTo: selectbox.header,
						defaultType: 'container',
						cls: 'sb-library-panel',
						items: [{
							cls: 'sb-gradient-dialog-color-panel',
							items: colorDropdown
						}, {
							cls: 'sb-library-panel-separator'
						}, {
							cls: 'sb-filter-by-direction',
							defaultType: 'checkbox',
							items: [{
								xtype: 'label',
								text: Sb.Locale.get('filterByDirection') + ':',
								cls: 'sb-filter-by-direction-title'
							}, {
								id: 'filter-vertical',
								boxLabel: Sb.Locale.get('vertical'),
								ref: '../filterVertical',
								checked: true,
								handler: function (me) {
									if (!selectbox.settingsForm.filterHorizontal.getValue()) {
										me.setValue(true);
									}
									selectbox.doFilter();
								}
							}, {
								id: 'filter-horizontal',
								boxLabel: Sb.Locale.get('horizontal'),
								ref: '../filterHorizontal',
								checked: true,
								handler: function (me) {
									if (!selectbox.settingsForm.filterVertical.getValue()) {
										me.setValue(true);
									}
									selectbox.doFilter();
								}
							}]
						}, {
							cls: 'sb-library-panel-separator'
						}, {
							cls: 'sb-invert-colors',
							defaultType: 'radio',
							items: [{
								xtype: 'label',
								text: Sb.Locale.get('invertBorderColors') + ':',
								cls: 'sb-invert-colors-title'
							}, {
								id: 'invert-color-black',
								name: 'invertBackgroundColors',
								boxLabel: Sb.Locale.get('black'),
								checked: true,
								listeners: {
									scope: selectbox,
									check: selectbox.doFilter
								}
							}, {
								id: 'invert-color-white',
								name: 'invertBackgroundColors',
								boxLabel: Sb.Locale.get('white')
							}]
						}]
					});
					selectbox.assetHeight += selectbox.header.getHeight();
					selectbox.doFilter();
				},
				beforecollapse: function(selectbox) {
					selectbox.settingsForm.filterHorizontal.reset();
					selectbox.settingsForm.filterVertical.reset();
				}
			}
		});
	},

	createPatternField: function () {
		return new Sb.form.SelectBox({
			xtype: 'sbselectbox',
			cls: 'sb-colordropdown-background-selectbox',
			triggerClass: 'sb-background-trigger',
			triggerAction: 'all',
			mode: 'local',
			width: 238,
			valueField: 'name',
			lazyInit: false,
			itemTools: {
				'delete': {
					scope: this,
					isAvailable: function (itemData) {
						return itemData.isCustom;
					},
					handler: function (selectbox, record) {
						selectbox.dialog = Ext.Msg.show({
							title: Sb.Locale.get('dialogTitleDeleteBackground'),
							msg: Sb.Locale.get('dialogMsgConfirmDeleteBackground'),
							closable: false,
							buttons: Ext.Msg.YESNO,
							scope: this,
							fn: function (btn) {
								if (btn == 'yes') {
									this.onDeleteBackground(selectbox, record);
								}
							}
						});
					}
				}
			},
			headerCfg: {
				layout: 'hbox',
				layoutConfig: {
					pack: 'center'
				}
			},
			header: [
				this.createColorDropdown({
					getMenuParent: function () {
						return this.el.up('.sb-selectbox-list') || document.body;
					}
				}), {
				xtype: 'form',
				margins: '0 0 0 10',
				fileUpload: true,
				border: false,
				hideLabels: true,
				items: [{
					xtype: 'fileuploadfield',
					regex: new RegExp('\\.(' + Sb.Config.imageValidExtensions.join('|') + ')$', 'i'),
					regexText: Sb.Locale.get('imageUploadErrorTypeInvalid'),
					buttonOnly: true,
					buttonText: Sb.Locale.get('uploadButton'),
					hideLabel: true,
					listeners: {
						scope: this,
						fileselected: this.onUploadBackground
					}
				}]
			}],
			itemRenderer: Ext.createDelegate(function(item) {
				var url;

				if (!item.isCustom) {
					url = Sb.Config.templatesUrl + '/' + Sb.Editor.model.data.template.code
						+ '/decorations/background/' + item.name + '/images/' + item.name + '.png';
				} else {
					url = Sb.Editor.model.design.getBackgroundUrl(this.uploadParams.container) + '/' + item.name;
				}

				return '<div style="background: url(' + url
					+ ') no-repeat center;background-size: contain;height: 100%;"></div>';
			}, this),
			store: {
				xtype: 'arraystore',
				id: 'backgrounds',
				fields: ['name', 'isCustom'],
				sortInfo: {
					field: 'isCustom',
					direction: 'DESC'
				},
				loadCustomBackgrounds: function () {
					Sb.SiteService.getBackgrounds(function (result) {
						var backgrounds = [];
						Ext.each(result, function (item) {
							if (this.find('name', item) === -1) {
								backgrounds.push([item, true]);
							}
						}, this);
						this.loadData(backgrounds, true);
						this.multiSort([this.sortInfo]);
					}, this);
				},
				listeners: {
					remove: function (store, record) {
						Sb.SiteService.deleteBackground(record.get('name'));
					}
				}
			},
			listeners: {
				scope: this,
				select: function (combo, record) {
					this.backgroundValue.pattern = record.data.name;
					this.fireEvent('backgroundchange', this, this.backgroundValue);
				}
			}
		});
	},

	createImageField: function () {
		return new Sb.form.HeaderSelectBox({
			filter: function (record) {
				return record.get('name') !== 'fallback' && !Sb.utils.Flash.isFlash(record.get('name'));
			},
			validExtensions: Sb.Config.imageValidExtensions,
			imageUploadErrorMessage: Sb.Locale.get('imageUploadErrorTypeInvalid'),
			listeners: {
				scope: this,
				select: function (select, record) {
					this.headerSelect(record);
				},
				deleteheader: function () {
					var record = this.backgroundImage.store.getAt(this.backgroundImage.selectedIndex);
					this.headerSelect(record);
				}
			}
		});
	},

	headerSelect: function (record) {
		if (record.get('isCustom')) {
			this.backgroundValue.image = 'Header/' + record.get('name') + '?' + record.get('time');
		} else {
			this.backgroundValue.image = record.get('name');
		}
		this.fireEvent('backgroundchange', this, this.backgroundValue);
	},

	updateColorButtons: function () {
		if (!this.colorType.rendered) {
			return;
		}

		var customColorButton = this.colorType.el.child('.sb-color-dropdown-custom-button'),
			color = this.value || '#000000';

		if (color.indexOf('color') === 0) {
			color = Sb.Editor.model.design.getColor(color);
		}

		customColorButton.setStyle('background-color', color);
	},

	setValue: function (color) {
		if (color === this.value) {
			return;
		}
		this.setRawValue(color);
		this.fireEvent('change', this, color);
	},

	setRawValue: function (color) {
		var label;

		if (color === this.value) {
			return;
		}

		this.value = color;

		if (color == 'transparent') {
			this.colorType.setValue('transparent');
			this.colorField.setVisible(false);
			this.colorPalette.clear();
		} else if (color == 'inherit') {
			this.colorType.setValue('inherit');
			this.colorField.setVisible(false);
			this.colorPalette.clear();
		} else {
			var isColorValid = Ext.ux.util.Color.isValid(color);
			if (!isColorValid && this.colorPalette) {
				this.colorType.setValue('palette');
				this.colorField.setVisible(false);
				this.colorPalette.select(color, true);
				color = Sb.Editor.model.design.getColor(color);
			} else {
				this.colorType.setValue('custom');
				this.colorField.setVisible(true);
				this.colorPalette.clear();
			}
			this.colorField.setRawValue(color, isColorValid);
			this.picker.setRawValue(color);
			this.updateColorButtons();
		}

		if (this.backgroundGradient) {
			this.backgroundGradient.setBackgroundColor(color);
		}

		Ext.each(this.colorDropdowns, function (field) {
			field.setValue(this.value);
		}, this);

		if (this.allowBackground) {
			this.backgroundPattern.setBackgroundColor(color);
		}

		if (this.allowStroke) {
			this.strokeField.setDisabled(this.value == 'inherit');
			label = Sb.Locale.get('stroke' + (this.value == 'inherit' ? 'Inherit' : ''));
			if (this.strokeField.rendered) {
				this.strokeField.wrap.child('.x-form-cb-label').update(label);
			} else {
				this.strokeField.boxLabel = label;
			}
		}
	},

	onColorTypeChange: function (group, radio) {
		switch (radio.getGroupValue()) {
			case 'transparent':
				this.setValue('transparent');
				break;

			case 'inherit':
				this.setValue('inherit');
				break;

			case 'palette':
				this.setValue(this.colorPalette.value || 'color1');
				break;

			case 'custom':
				this.setValue(this.picker.getValue());
				break;
		}
	},

	onBackgroundTypeChange: function (group, radio) {
		var type = radio.getGroupValue();

		this.backgroundValue.type = type;
		this.refreshBackground();
		this.fireEvent('backgroundchange', this, this.backgroundValue);
	},

	refreshBackground: function() {
		var type = this.backgroundValue.type;

		this.colorType.setVisible(type === 'none');
		this.backgroundGradient.setVisible(type === 'gradient');
		this.backgroundPattern.setVisible(type === 'pattern');
		this.backgroundPosition.setVisible(['pattern', 'image'].indexOf(type) !== -1);
		this.backgroundStretchType.setVisible(['pattern', 'image'].indexOf(type) !== -1);
		this.backgroundImage.setVisible(type === 'image');
		this.backgroundColorWrapper.setVisible(type == 'gradient' || type == 'pattern');
	},

	setAllowBlank: function(value) {
		this.allowBlank = value;
		this.colorType.eachItem(function(item){
			if (item.inputValue == 'transparent') {
				item.setVisible(value);
			}
		});
		Ext.each(this.colorDropdowns, function (field) {
			field.setAllowBlank(value);
		});
	},

	setAllowHeaders: function (value) {
		this.allowHeaders = value;
		if (this.backgroundType.rendered) {
			this.backgroundType.items.itemAt(3).setVisible(value);
		} else {
			this.backgroundType.items[3].hidden = !value;
		}
	},

	setParentColor: function (value) {
		this.parentColor = value;
		if (this.allowInherit && this.colorType.rendered) {
			this.colorType.el.child('.sb-color-dropdown-inherit-button')
				.setStyle('background-color', Sb.Editor.model.design.getColor(this.parentColor));
		}
		Ext.each(this.colorDropdowns, function (field) {
			field.setParentColor(value);
		});
	},

	setVisibleOpacity: function(value) {
		if (this.allowOpacity) {
			this.opacityField.setVisible(!!value);
		}
	},

	setBackgroundValue: function (background) {
		var addNewPattern = function (pattern, store) {
			if (store.find('name', pattern) === -1) {
				store.addSorted(new store.recordType(
					{ name: pattern, isCustom: true }
				));
			}
		};
		if (!this.allowBackground) {
			return;
		}

		this.backgroundValue = background;

		this.backgroundType.suspendEvents();
		this.backgroundType.setValue(this.backgroundValue.type);
		this.backgroundType.resumeEvents();

		if (this.backgroundValue.type === 'pattern' && this.backgroundValue.pattern !== '') {
			if (this.backgroundPattern.store.getCount() > 0
				&& this.backgroundPattern.store.find('name', this.backgroundValue.pattern) === -1) {
				addNewPattern(this.backgroundValue.pattern, this.backgroundPattern.store);
			} else {
				this.backgroundPattern.store.addListener('load',
					addNewPattern.createCallback(this.backgroundValue.pattern, this.backgroundPattern.store),
					this, {single: true});
			}
		}

		this.backgroundPattern.setValue(this.backgroundValue.pattern);
		this.backgroundGradient.setValue(this.backgroundValue.gradient);
		Ext.each(this.colorDropdowns, function (field) {
			field.setValue(this.value);
		}, this);

		this.backgroundPosition.setValue(this.backgroundValue.position);
		this.backgroundStretchType.setValue(this.backgroundValue.stretch);
		if (this.rendered) {
			this.refreshBackground();
		}

		if (this.allowOpacity) {
			this.opacityField.setValue(this.backgroundValue.opacity);
		}
	},

	onUploadBackground: function (field) {
		if (!field.isValid()) {
			Ext.Msg.show({
				title: Sb.Locale.get('imageUploadErrorDialogTitle'),
				msg: field.getActiveError(),
				closable: false,
				buttons: Ext.Msg.OK
			});
			field.resetOnUpload();
			return;
		}

		field.ownerCt.getForm().submit({
			url: this.uploadUrl,
			params: this.uploadParams,
			waitMsg: Sb.Locale.get('uploadingImage'),
			scope: this,
			success: function (form, action) {
				if (action.result.width * action.result.height > 300000) {
					Sb.Editor.showLocalizedWarning('backgroundUploadWarningImageTooLarge');
				}

				this.backgroundPattern.collapse();
				this.backgroundValue.pattern = action.result.uploadedFileName;
				this.backgroundPattern.store.addSorted(new this.backgroundPattern.store.recordType(
					{ name: this.backgroundValue.pattern, isCustom: true }
				));
				this.backgroundPattern.setValue(this.backgroundValue.pattern);
				field.resetOnUpload();
				this.fireEvent('backgroundchange', this, this.backgroundValue);
			},
			failure: function (form, action) {
				if (action.result.message) {
					Ext.Msg.show({
						title: Sb.Locale.get('imageUploadErrorDialogTitle'),
						msg: Sb.Locale.get(action.result.message),
						closable: false,
						buttons: Ext.Msg.OK
					});
				}
				field.resetOnUpload();
			}
		});
	},

	show: function () {
		this.callParent(arguments);
		this.isCancelled = false;
		this.initialValue = this.value;
		this.initialBackgroundValue = Ext.apply({}, this.backgroundValue);
		this.initialStrokeValue = this.allowStroke ? this.strokeField.getValue() : undefined;
		if (this.allowBackground) {
			this.refreshBackground();
		}

		if (this.allowHeaders) {
			this.setInitialValueBackgroundImage();
		}
		this.setAllowBlank(this.allowBlank);
		this.setParentColor(this.parentColor);
	},

	setInitialValueBackgroundImage: function () {
		var startIndex, imageName, imagePath;

		if (this.backgroundValue.image.indexOf('.') === -1) {
			imagePath = Sb.Config.headersUrl + '/' + this.backgroundValue.image + '/preview.' +
				Sb.HeaderManager.getHeaderFormat(this.backgroundValue.image);
		} else if (this.backgroundValue.image.indexOf('builtin/') !== -1) {
			startIndex = this.backgroundValue.image.lastIndexOf('/');
			imageName = this.backgroundValue.image.substr(
				startIndex, this.backgroundValue.image.lastIndexOf('.') - startIndex
			);
			imagePath = Sb.Config.headersUrl + '/' + imageName + '/preview.' +
				this.backgroundValue.image.substr(this.backgroundValue.image.lastIndexOf('.') + 1);
		} else {
			imagePath = Sb.Editor.model.design.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder +
				'/' + this.backgroundValue.image.replace('image_', 'thumbnail_image_');
		}
		this.backgroundImage.setValue(imagePath);
	}
});