/**
 * Class for Columns widget model
 *
 * @class Sb.model.ColumnsWidget
 * @extends Sb.model.Widget
 */
Ext.define('Sb.model.ColumnsWidget', {
	extend: 'Sb.model.Widget',

	constructor: function (config) {
		config = Ext.apply({
			type: 'columns'
		}, config);

		Sb.model.ColumnsWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Add a new column in columns.
	 * @param {Number} position
	 * @param {Sb.model.Widget|Sb.widgetType.Base} child Widget that should be added into created column
	 */
	addColumn: function (position, child) {
		this.createChild(Sb.WidgetTypeManager.get('column_container'), position, function (columnModel) {
			this.onColumnAdd(columnModel);
			if (child instanceof Sb.model.Widget) {
				columnModel.insert(0, child);
			} else if (child instanceof Sb.widgetType.Base) {
				columnModel.createChild(child, 0);
			}
		}, this);
	},

	/**
	 * Move all the widgets from his column in the specified container from specified position
	 * @param {Sb.model.ColumnsColumnWidget} column The column from which widgets should be moved
	 * @param {Sb.model.Widget} [container] Container to insertion
	 * @param {Number} [position] Start position in a container to insertion
	 * @return {Object} Information on insertion. {position: position to insertion, container: Container to insertion}
	 */
	moveAllFromColumn: function (column, container, position) {
		if (undefined === container) {
			container = this.ownerWidget;
			if (undefined === position) {
				position = container.widgets.indexOf(this) + 1;
			}
		}

		if (undefined === position) {
			position = container.widgets.getCount();
		}

		var positionInColumn, widgetInColumn = null;
		for (positionInColumn = column.widgets.getCount() - 1; positionInColumn >= 0; positionInColumn--) {
			widgetInColumn = column.widgets.item(positionInColumn);
			container.insert(position, widgetInColumn);
		}
		return {position: position, container: container};
	},

	/**
	 *
	 * @param {Sb.model.Widget} column Column model for change width
	 * @param {Number} newWidth New width of column
	 * @param {Boolean} noSaveChanges
	 */
	setColumnWidth: function (column, newWidth, noSaveChanges) {
		var columnPosition = this.widgets.indexOf(column);
		if (columnPosition == -1 || this.widgets.last() == column) {
			return;
		}
		var oldWidth = parseFloat(column.get('width')),
			nextColumn = this.widgets.item(columnPosition + 1),
			deltaWidth = Ext.util.Format.round(newWidth - oldWidth, 3),
			nextColumnOldWidth,
			nextColumnNewWidth;
		column.set('width', newWidth + '%');
		nextColumnOldWidth = parseFloat(nextColumn.get('width'));
		nextColumnNewWidth = Ext.util.Format.round(nextColumnOldWidth - deltaWidth, 3);
		nextColumn.set('width', nextColumnNewWidth + '%');

		if (noSaveChanges !== false) {
			this.onSaveColumnsWidth();
		}
	},

	onSaveColumnsWidth: function () {
		this.widgets.each(function (column) {
			column.save();
		});
	},

	onColumnAdd: function (column) {
		var defaultColumnWidth = Ext.util.Format.round(100 / this.widgets.getCount(), 0);

		column.set('width', defaultColumnWidth + '%');
		this.widgets.each(function (columnWidgetModel, position) {
			if (column != columnWidgetModel) {
				var oldWidth = parseFloat(columnWidgetModel.get('width')),
					newWidth = oldWidth - Ext.util.Format.round(oldWidth * defaultColumnWidth / 100, 3);
				columnWidgetModel.set('width', newWidth + '%');
			}
		}, this);
		this.onSaveColumnsWidth();
	},

	onWidgetRemove: function (widget) {
		Sb.model.ColumnsWidget.superclass.onWidgetRemove.call(this, widget);
		var removedColumnWidth = parseFloat(widget.get('width'));
		var delta = Ext.util.Format.round(removedColumnWidth / this.widgets.getCount(), 3);
		var lastDelta = removedColumnWidth - delta * (this.widgets.getCount() - 1);

		this.widgets.each(function (column, index, count) {
			var oldWidth = parseFloat(column.get('width')),
				newWidth = 0;
			if (index == count - 1) {
				newWidth = oldWidth + lastDelta;
			} else {
				newWidth = oldWidth + delta;
			}
			column.set('width', newWidth + '%');
		}, this);
		this.onSaveColumnsWidth();
	}
});