/**
 * @class Sb.model.NavigationWidget
 * @extends Sb.model.Widget
 * Class for navigation model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Ext.define('Sb.model.NavigationWidget', {
	extend: 'Sb.model.Widget',

	/**
	 * @cfg {String} type
	 * defaults to <tt>navigation</tt>
	 */

	constructor: function (config) {
		config = Ext.apply({
			type: 'navigation'
		}, config);

		this.addEvents(
			/**
			 * @event levelchange
			 * Fires after the levels/depth has changed.
			 */
			'levelchange',
			/**
			 * @event fontchange
			 * Fires after the link font size/family has changed.
			 */
			'fontchange',
			/**
			 * @event alignmentchange
			 * Fires after the menu alignment has changed.
			 */
			'alignmentchange',
			/**
			 * @event colorchange
			 * Fires after the item colors/decorations has changed.
			 */
			'colorchange',
			/**
			 * @event decorationchange
			 * Fires after the item padding, shadows, etc has changed
			 */
			'decorationchange',
			/**
			 * @event backgroundchange
			 * Fires after the item background has changed
			 */
			'backgroundchange'
		);

		Sb.model.NavigationWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Level-independent getter.
	 *
	 * @param {String} propname
	 * @param {Number} [level] When it`s undefined, it will returns prop for level 1.
	 */
	getByLevel: function (propname, level) {
		if (undefined === level) {
			level = 1;
		}
		return this.data['level' + level + propname];
	},

	/**
	 * Level-independent setter.
	 *
	 * @param {String} propname
	 * @param {String|Number} propvalue
	 * @param {Number} [level] When it`s undefined, it will set props on all levels 1 and 2.
	 */
	setByLevel: function (propname, propvalue, level) {
		var levels = (undefined === level) ? [1, 2] : [level];
		Ext.each(levels, function (level) {
			this.set('level' + level + propname, propvalue);
		}, this);
	},

	/**
	 * Fires on changing of single property change to allow us fire up navigation widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		if (key == 'level' || key == 'depth') {
			this.fireEvent('levelchange');
		} else if (/^level[12](Font(Family|Size)|Bold|Italic|Underline)/.test(key)) {
			this.fireEvent('fontchange');
		} else if (/^level[12]Alignment$/.test(key)) {
			this.fireEvent('alignmentchange');
		} else if (/(padding|margin|shadow|corners|border)(?!color)/i.test(key)) {
			this.fireEvent('decorationchange', this);
		} else if (/background(?!color)/i.test(key)) {
			this.fireEvent('backgroundchange', this);
		} else {
			this.fireEvent('colorchange');
		}
	}
});
