/**
 * @class Sb.model.RemotingProvider
 * @extends Ext.direct.RemotingProvider
 */
Ext.define('Sb.model.RemotingProvider', {
	extend: 'Ext.direct.RemotingProvider',

	constructor: function (config) {
		Sb.model.RemotingProvider.superclass.constructor.call(this, config);

		this.on({
			beforecall	: this.onBeforeCall,
			data		: this.onReceiveData,
			scope		: this
		});

		/**
		 * Counts calls of API methods.
		 *
		 * @type Number
		 * @property callCounter
		 */
		this.callCounter = 0;

		this.addEvents(
			/**
			 * Fires when last call, from all previously started, will be finished.
			 *
			 * @event afterlastcall
			 */
			'afterlastcall'
		);
	},

	/**
	 * Runs after last call of all currently active asynchronious calls.
	 *
	 * @param {Function} fn Callback, that will be called immediatelly or after last call.
	 * @param {Object} scope Scope of the callback
	 */
	runAfterLastCall: function (fn, scope) {
		if (0 === this.callCounter) {
			Ext.callback(fn, scope);
		} else {
			this.on('afterlastcall', fn, scope, {single: true});
		}
	},

	/**
	 * Fires before each remote method call.
	 *
	 * See {@link Ext.direct.RemotingProvider#event-beforecall}
	 */
	onBeforeCall: function (me, transaction, meta) {
		this.callCounter++;
	},

	/**
	 * Fires after each remote method data receive.
	 *
	 * See {@link Ext.direct.RemotingProvider#event-data}
	 */
	onReceiveData: function (me, e) {
		this.callCounter--;
		if (0 === this.callCounter) {
			this.fireEvent('afterlastcall');
		}
		if (e.redirect) {
			window.location = e.redirect;
			return;
		}

		var message		= '',
			format		= Ext.util.Format,
			encode		= format.htmlEncode,
			nl2br		= format.nl2br,
			key			= Sb.Locale.get,
			exceptions	= Ext.Direct.exceptions;

		if ('exception' == e.type || e.isLogErrors) {
			Sb.Loading.hide();
			Ext.getBody().unmask();
			switch (e.code) {
				case exceptions.TRANSPORT:
					message = encode(key('UnableConnectToServerErrorMessage', Sb.Config.general.product));
					break;
				case exceptions.PARSE:
					message = encode(key('InvalidServerResponseErrorMessage'));
					break;
				default:
					message = encode(key('GenericServerErrorMessage')) +
								'<div class="sb-error-details">' + encode(e.message) + '</div>';
					if (e.file && e.line) {
						message += '<div class="sb-error-details">in ' + encode(e.file) +
							' on line ' + e.line + '</div>';
					}
					if (e.where) {
						message += '<div id="sb-error-stacktrace"></div>';
					}
					break;
			}
			Ext.Msg.show({
				width	: (e.where) ? 1200 : 600,
				maxWidth: 1200,
				title	: key('errorMessageBoxTitle'),
				msg		: message,
				closable: false,
				buttons	: Ext.Msg.OK,
				icon	: Ext.Msg.ERROR
			});

			if (!e.where) {
				return;
			}
			var traceData = [];
			Ext.iterate(e.where, function (traceItem) {
				traceData.push([
					traceData.length,
					traceItem['class'],
					traceItem['function'],
					encode(
						traceItem.file ||
							(traceItem['class'] ? traceItem['class'] + '->' : '') + traceItem['function'] + '(args)'
					),
					traceItem.line,
					traceItem.args
				]);
			}, this);

			var expander = new Ext.ux.grid.RowExpander({
				tpl: new Ext.XTemplate(
					'<tpl if="file">' +
						'<p><b>File:</b> {[values.file]}</p>' +
					'</tpl>' +
					'<tpl if="line">' +
						'<p><b>Line:</b> {[values.line]}</p>' +
					'</tpl>' +
					'<tpl if="functionName">' +
						'<p><b>Method:</b> ' +
							'<tpl if="className">{[Ext.util.Format.htmlEncode(values.className)]}-&gt;</tpl>' +
							'{[Ext.util.Format.htmlEncode(values.functionName)]}' +
							'({[Ext.util.Format.htmlEncode(this.renderArgs(values.args))]})' +
						'</p>' +
					'</tpl>',
					{
						renderArgs: function (args) {
							var formattedValue, result = [];
							Ext.iterate(args, function (arg, index) {
								if (Ext.isObject(args)) {
									// :INFO: abelyaev 20140602: swtich vars because of Ext.iterate behaviour
									var tmp = index;
									index = arg;
									arg = tmp;
								}
								switch (arg[0]) {
									case 'object':
										formattedValue = String.format('object({0})', arg[1]);
										break;
									case 'array':
										formattedValue = String.format(
											'array({0})',
											(Ext.isArray(arg[1]) || Ext.isObject(arg[1]))
												? this.renderArgs(arg[1])
												: arg[1]
										);
										break;
									case 'string':
										formattedValue = String.format('"{0}"', arg[1]);
										break;
									case 'null':
										formattedValue = 'null';
										break;
									case 'boolean':
										formattedValue = arg[1] ? 'true' : 'false';
										break;
									case 'resource':
										formattedValue = 'resource';
										break;
									default:
										formattedValue = arg[1];
										break;
								}
								result.push(
									Ext.isNumber(index)
										? formattedValue
										: String.format('"{0}" => {1}', index, formattedValue)
								);
							}, this);

							return result.join(', ');
						}
					}
				)
			});
			var grid = new Ext.grid.GridPanel({
				store: new Ext.data.ArrayStore({
					fields: ['number', 'className', 'functionName', 'file', 'line', 'args'],
					data: traceData
				}),
				cm: new Ext.grid.ColumnModel({
					columns: [
						expander,
						{header: '#', width: 10, dataIndex: 'number'},
						{header: 'Where', width: 500, dataIndex: 'file'},
						{header: 'Line', width: 20, dataIndex: 'line'}
					]
				}),
				viewConfig: {
					forceFit: true
				},
				width: 1125,
				height: 500,
				plugins: expander,
				collapsible: false,
				animCollapse: false,
				title: encode(key('Stacktrace')),
				renderTo: 'sb-error-stacktrace'
			});
		}
	}
});
Ext.Direct.PROVIDERS['sbremoting'] = Sb.model.RemotingProvider;