/**
 * @class Sb.model.SiteDesign
 * @extends Sb.model.Node
 * Class for site design model
 * @constructor
 * @param {Object} config The configuration options for the site design model
 */
Ext.define('Sb.model.SiteDesign', {
	extend: 'Sb.model.Node',

	/**
	 * @type {String}
	 */
	presetType: 'generic',

	/**
	 * @type {String}
	 */
	presetName: '',

	/**
	 * Need for checks is user backgrounds during design preview
	 * @type {Object}
	 */
	isUserBackground: {},

	constructor: function (config) {
		this.api = {
			load: Sb.SiteService.getDesignSettings,
			save: Sb.SiteService.setDesignSettings
		};

		this.addEvents(
			/**
			 * @event decorationchange
			 * Fires after the decoration has changed.
			 * @param {String} decorationType
			 * @param {String} decoration Name of decoration
			 */
			'decorationchange',
			/**
			 * @event colroschemechange
			 * Fires after the color scheme has changed.
			 */
			'colorschemechange',
			/**
			 * @event layoutchange
			 * Fires after the layout has changed.
			 */
			'layoutchange',
			/**
			 * @event fontchange
			 * Fires after the font has changed.
			 */
			'fontchange',
			/**
			 * @event backgroundcolorchange
			 * Fires after the background colors has changed.
			 */
			'backgroundcolorchange',

			/**
			 * @event pagelayoutchange
			 * Fires after the page layout has changed.
			 */
			'pagelayoutchange'
		);

		Sb.model.SiteDesign.superclass.constructor.call(this, config);

		this.contrastModel = [
			[-0.2, -0.25], // shadow 1
			[-0.4, -0.5], // shadow 2
			[-0.1, 0.25], // light 1
			[-0.2, 0.5] // light 2
		];
	},

	/**
	 * Return design settings
	 *
	 * @return Sb.model.SiteDesign.data
	 */
	getSettings: function () {
		return this.data;
	},

	/**
	 * Return list of used gradients
	 *
	 * @return {Array} Like ["background22-black", "background5-black"]
	 */
	getUsedGradients: function () {
		var list = [];
		Ext.iterate(this.data.background, function (property, value) {
			if (Sb.Utils.getBackgroundType(value).value === 'gradient') {
				list.push(value);
			}
		}, this);
		return list;
	},

	/**
	 * Collection of regular expressions, that helps us to check for decorators from decorations
	 *
	 * @var Object
	 */
	reIsDecorator: {
		background: [new RegExp('^.+Background$'), new RegExp('^background[^\\.]+$')]
	},

	/**
	 * Determines whether decoration is decorator.
	 * For example in background property with file name isn`t decorator, as it didn`t has CSS file associated.
	 *
	 * @param decorationType
	 * @param property
	 * @returns {Boolean}
	 */
	isDecorator: function (decorationType, property) {
		var reIsGood = this.reIsDecorator[decorationType];

		return reIsGood && reIsGood[0].test(property) && reIsGood[1].test(this.data[decorationType][property]);
	},

	/**
	 * Apply new decoration
	 *
	 * @param {String} decorationType Decoration type
	 * @param {String} property Decoration name
	 * @param {String} decoration Decoration value
	 */
	setDecoration: function (decorationType, property, decoration) {
		var properties = {};
		properties[property] = decoration;

		return this.unserializeObject(decorationType, properties);
	},

	/**
	 * Get current decoration
	 *
	 * @param {String} decorationType Decoration type
	 * @param {String} property Decoration name
	 */
	getDecoration: function (decorationType, property) {
		if (undefined !== this.data[decorationType]) {
			return this.data[decorationType][property];
		}
		return undefined;
	},

	serialize: function () {
		var data;

		data = Sb.model.SiteDesign.superclass.serialize.call(this);
		data.layoutWidget = Sb.Editor.model.rootPage.widget.serialize();

		data.presetType = this.presetType;
		data.presetName = this.presetName;

		return data;
	},

	/**
	 * Unserialize design`s data in special order: colorScheme should be always first
	 *
	 * @param {Object} data May have key rootData
	 * @return {Boolean} Result of unserializing
	 */
	unserialize: function (data) {
		var isChanged = false,
			parentUnserialize = Sb.model.SiteDesign.superclass.unserialize,
			isLayoutChanged = false;

		if (undefined !== data.presetType) {
			this.presetType = data.presetType;
		}
		if (undefined !== data.presetName) {
			this.presetName = data.presetName;
		}
		delete data.presetType;
		delete data.presetName;

		if (data.layoutWidget && data.layoutWidget.uuid !== Sb.Editor.model.rootPage.widget.get('uuid')) {
			// Forget about the old user backgrounds in the new preview
			this.isUserBackground = {};

			Sb.Editor.model.rootPage.widget.unserialize(data.layoutWidget);
			isLayoutChanged = true;

			if (data.banner) {
				// unserialize from legacy preset with banner in content
				this.insertBanner(data.banner);
			}
		}
		delete data.layoutWidget;
		delete data.banner;

		if (data.colorScheme) {
			isChanged = parentUnserialize.call(this, {colorScheme: data.colorScheme}) || isChanged;
			delete data.colorScheme;
		}

		isChanged = parentUnserialize.call(this, data) || isChanged;

		if (!isLayoutChanged && isChanged && data.layout) {
			Ext.iterate(data.layout, function (key) {
				if (['headerLayout', 'contentLayout', 'footerLayout'].indexOf(key) != -1) {
					isLayoutChanged = true;
					return false;
				}
			});
		}

		if (isLayoutChanged) {
			this.fireEvent('pagelayoutchange');
		}

		return isChanged;
	},

	/**
	 * @param banner
	 * @private
	 */
	insertBanner: function (banner) {
		var widgets = Sb.Editor.model.rootPage.widget.widgets.get('content').widgets;
		widgets.insert(
			widgets.indexOf(
				widgets.find(function (widget) {
					return widget.get('type') === 'pagecontent';
				})
			),
			new Sb.model.HeaderWidget(banner)
		);
	},

	/**
	 * Unserializes objects
	 * @param {String} category
	 * @param {Object} properties
	 */
	unserializeObject: function (category, properties) {
		var isChanged		= false,
			isUpdated		= this.data[category] !== undefined;

		if (!isUpdated) {
			this.data[category] = {};
		}

		Ext.iterate(properties, function (property, value) {
			var oldValue = this.data[category][property];
			if (oldValue !== value) {
				this.data[category][property] = value;
				isChanged = true;
				if ('font' == category) {
					if ('textFont' == property || 'textFontSize' == property) {
						this.fireEvent('propertychange', category + '.' + property, value, oldValue);
					}
				}
				if (category == 'background' || category == 'border' || category == 'external-border') {
					this.fireEvent('decorationchange', category, value, property);
				}
			}
		}, this);

		if (isChanged) {
			this.fireEvent(category.toLowerCase() + 'change', this);
		}

		return isChanged;
	},

	/**
	 * Get number of columns in design
	 */
	countColumns: function () {
		return 1 +
			(this.data.layout.sidebarFirst ? 1 : 0) +
			(this.data.layout.sidebarSecond ? 1 : 0);
	},

	getSidebarProperty: function (sidebar, property) {
		return this.data.layout.sidebarFirstPosition == sidebar ?
			this.data.layout['sidebarFirst' + property] :
			this.data.layout['sidebarSecond' + property];
	},

	isLeftColumnVisible: function () {
		return (this.data.layout.sidebarFirstPosition == 'left' && this.data.layout.sidebarFirst)
			|| (this.data.layout.sidebarSecondPosition == 'left' && this.data.layout.sidebarSecond);
	},

	isRightColumnVisible: function () {
		return (this.data.layout.sidebarFirstPosition == 'right' && this.data.layout.sidebarFirst)
			|| (this.data.layout.sidebarSecondPosition == 'right' && this.data.layout.sidebarSecond);
	},

	isSidebarsSwitched: function () {
		return this.data.layout.sidebarFirstPosition == 'right';
	},

	isPressColumnsToBottom: function () {
		return this.data.layout.pressColumnsToBottom == 1;
	},

	save: function (callbacks) {
		if (this.presetType !== 'generic') {
			// after saving design a preset type is always "generic"
			callbacks.success = Ext.createDelegate(function (successCallback) {
				this.presetType = 'generic';
				this.presetName = '';
				Ext.callback(successCallback);
			}, this, [callbacks.success || Ext.emptyFn]);
		}
		Sb.model.SiteDesign.superclass.save.call(this, callbacks);
	},

	getResourcesUrl: function () {
		var templatePath = Sb.Config.templatesUrl + '/' + Sb.Editor.model.get('template').code;

		if (this.presetType === 'special') {
			return templatePath + '/presets/' + this.presetName;
		}

		if (this.presetType === 'default') {
			return templatePath + '/designs/' + this.presetName;
		}

		return Sb.Editor.model.getUrl();
	},

	/**
	 * Returns background url without file name
	 * @param {String} region
	 * @return {String}
	 */
	getBackgroundUrl: function (region) {
		var baseUrl = this.getResourcesUrl();
		if (this.isUserBackground[region]) {
			baseUrl = Sb.Editor.model.getUrl();
		}

		return  baseUrl + '/' + Sb.Config.path.attachmentsFolder + '/' + Sb.Config.path.backgroundFolder;
	},

	/**
	 * Returns background properties for region
	 * @param {String} region
	 * @returns {Object}
	 */
	getBackground: function (region) {
		var background = this.data.background[region + 'Background'],
			type = Sb.Utils.getBackgroundType(background).value;

		return {
			type: type,
			gradient: type === 'gradient' ? background : 'background10-black',
			pattern: type === 'pattern' ? background : 'background1',
			image: type === 'image' ? background : 'academic_01',
			position: this.data.background[region + 'BackgroundPosition'] || 'top left',
			stretch: ['pattern', 'image'].indexOf(type) !== -1 ?
				this.data.background[region + 'BackgroundStretch'] : 'tile',
			opacity: this.data.background[region + 'BackgroundOpacity'],
			color: this.data.backgroundColor[region + 'Color']
		};
	},

	/**
	 * Set background properties for region
	 * @param {String} region
	 * @param {Object} background
	 * @return {Boolean}
	 */
	setBackground: function (region, background) {
		var backgroundProperties = {};
		if (background.type === 'gradient') {
			backgroundProperties[region + 'Background'] = background.gradient;
		} else if (background.type === 'pattern') {
			backgroundProperties[region + 'Background'] = background.pattern;
		} else if (background.type === 'image') {
			backgroundProperties[region + 'Background'] = background.image;
		} else {
			backgroundProperties[region + 'Background'] = '';
		}
		backgroundProperties[region + 'BackgroundPosition'] = background.position;
		backgroundProperties[region + 'BackgroundStretch'] = background.stretch;
		backgroundProperties[region + 'BackgroundOpacity'] = background.opacity;
		return this.unserializeObject('background', backgroundProperties);
	},

	deleteBackground: function (name) {
		var background = {};
		Ext.iterate(this.data.background, function (key, value) {
			if (value === name) {
				background[key] = '';
			}
		}, this);

		this.unserializeObject('background', background);
	},

	/**
	 * Convert scheme color to color
	 * @param {String} color
	 * @return {String}
	 */
	getColor: function (color) {
		var m = Ext.isString(color) && color.match(/^(color\d)(?:-(\d))?$/), variant;

		if (m) {
			color = this.get('colorScheme')[m[1]];

			if (m[2]) {
				variant = parseInt(m[2], 10) - 1;
				color = Ext.ux.util.Color.getVariant(
					color,
					this.contrastModel[variant][0],
					this.contrastModel[variant][1]
				);
			}
		}

		return color;
	},

	setContrastModel: function (contrastModel) {
		this.contrastModel = contrastModel;
		this.fireEvent('colorschemechange');
	}
});
