/**
 * @class Sb.utils.Tinymce
 * <p>Contains a utils functions for TinyMCE editor.</p>
 * @singleton
 */
Ext.define('Sb.utils.Tinymce', {
	singleton: true,

	/**
	 * Updates editor cached inline style by copying it from current element style
	 *
	 * @param {Ext.Element} el Element on which editor`s cached style value is updated [optional]
	 */
	cacheStyleInEditor: function (el) {
		el = el || this.el;
		var style = (Ext.isIE && !Ext.isIE9) ? el.dom.style.cssText : el.dom.getAttribute('style');
		el.set({
			'data-mce-style': style.toString()
		});
	},

	/**
	 * Get alignment of selected element
	 *
	 * @param {Ext.Element} el
	 * @return {String} alignment
	 */
	getAlignment: function (el) {
		var cssFloat = el.getStyle('float'),
			value = 'none';
		if ('left' == cssFloat || 'right' == cssFloat) {
			value = cssFloat;
		} else if ('block' == el.getStyle('display')) {
			value = 'center';
		}
		return value;
	},

	/**
	 * Parse JSON saved in attribute
	 *
	 * @param {Ext.Element} el
	 * @return {Object}
	 */
	parseJson: function (el) {
		return tinymce.util.JSON.parse(el.getAttribute('data-mce-json'));
	},

	/**
	 * Parse JSON saved in attribute
	 *
	 * @param {Ext.Element} el
	 * @param {Object} data
	 */
	saveJson: function (el, data) {
		el.set({'data-mce-json': tinymce.util.JSON.serialize(data, "'")});
	},

	/**
	 * Removes node from editor
	 *
	 * @param {Ext.Element} el
	 */
	removeFromEditor: function (el) {
		if (tinyMCE.activeEditor && el.dom) {
			tinyMCE.activeEditor.dom.remove(el.dom);
			delete el.dom;
		}
	},

	/**
	 * Selects node in editor
	 *
	 * @param {Ext.Element} el
	 */
	selectInEditor: function (el) {
		if (tinyMCE.activeEditor && el.dom) {
			tinyMCE.activeEditor.selection.select(el.dom);
		}
	},

	/**
	 * @param {tinymce.Editor} ed
	 * @returns {Node}
	 */
	getLinkNodeFromSelection: function (ed) {
		var selection = ed.selection,
			result;

		result = selection.getNode();
		result = ed.dom.getParent(result, "A");
		if (!result) {
			var rng = selection.getRng();
			if (rng.setStart && 'A' == rng.commonAncestorContainer) {
				result = rng.commonAncestorContainer;
			}
		}
		return result;
	},

	/**
	 * @param {tinymce.Editor} ed
	 * @param {Node} node
	 * @param {Object} bookmark
	 */
	removeNode: function (ed, node, bookmark) {
		bookmark = bookmark || ed.selection.getBookmark();
		if (node) {
			ed.execCommand("mceBeginUndoLevel");
			ed.dom.remove(node, 1);
			ed.execCommand("mceEndUndoLevel");
		}
		ed.selection.moveToBookmark(bookmark);
	},

	/**
	 * @param {tinymce.Editor} ed
	 * @param node
	 */
	selectNode: function (ed, node) {
		// Don't move caret if selection was image
		if (node.childNodes.length != 1 || node.firstChild.nodeName != 'IMG') {
			ed.focus();
			ed.selection.select(node);
			ed.selection.collapse(0);
			ed.windowManager.bookmark = ed.selection.getBookmark(1);
		}
	},

	/**
	 * @param {tinymce.Editor} ed
	 * @param {Object} bookmark
	 * @returns {Node}
	 */
	createLinkNode: function (ed, bookmark) {
		var result;

		ed.getDoc().execCommand("unlink", false, null);
		if (bookmark) {
			ed.selection.moveToBookmark(bookmark);
		}
		ed.execCommand("CreateLink", false, "#mctmp", {skip_undo : 1});

		result = ed.dom.select('a[href=#mctmp]')[0];
		if (!result) {
			ed.selection.setContent('<a href="#mctmp">' + ed.selection.getContent() + '</a>');
			result = ed.dom.select('a[href=#mctmp]')[0];
		}

		return result;
	}
});