/**
 * @class Sb.widget.ColumnContainer
 * @extends Sb.widget.Container
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.ColumnContainer', {
	extend: 'Sb.widget.Container',
	xtype: 'column_container_widget',

	autoEl : 'td',
	cls : 'widget-columns-column',

	/**
	 * Side paddings that define areas where dropped widget should create a column.
	 */
	sidePadding: 0.1,

	afterRender: function () {
		this.contentElement = this.el.insertFirst({
			cls: 'sb-widget-columns-column-content'
		});
		this.infoEl = this.el.insertFirst({
			cls: 'sb-widget-columns-column-info',
			cn: {}
		});
		this.infoEl.setDisplayed(false);
		Sb.widget.ColumnContainer.superclass.afterRender.call(this, arguments);
		// :INFO: smikhalyev 20120229: Configures the padding for the target zone in px.
		// See setPadding method's description in ExtJs DOC
		this.dt.setPadding(-10, 1, -10, 1);

		this.mon(this.model, 'propertychange', function (property, newValue, oldValue) {
			if (property == 'width') {
				this.onChangeWidth();
			}
		}, this);
		this.onChangeWidth();
	},

	getLayoutTarget: function () {
		return this.contentElement;
	},

	onChangeWidth: function () {
		this.setWidth(this.model.get('width'));
	},

	showInfo: function () {
		var width = Ext.util.Format.round(parseFloat(this.model.get('width')), 1),
			el = this.infoEl.first(),
			top = 10; // 10px offset from top of column for show width
		el.setLeftTop(Ext.util.Format.round((this.el.getWidth() - el.getWidth()) / 2, 0), top);
		el.update(width + '%');
		this.infoEl.setDisplayed(true);
	},

	hideInfo: function () {
		this.infoEl.setDisplayed(false);
	},

	/**
	 * Ability to add a new column
	 * @param {Sb.widgetType.Base} widgetType
	 * @return {Boolean}
	 */
	allowAddWidgetSide: function (widgetType) {
		if (!Sb.widget.ColumnContainer.superclass.allowAddWidgetSide.call(this, widgetType)) {
			return false;
		}
		var columns = this.parentWidget.columns.filterBy(function (columnWidget) {
			return !columnWidget.isHidden();
		});
		return columns.getCount() < Sb.WidgetTypeManager.get('columns').MAX_COLUMNS;
	},

	/**
	 * Check that current column is hidden
	 * @returns {Boolean}
	 */
	isHidden: function () {
		return this.getEl().getStyle('width') == 'auto';
	},

	/**
	 * Set hidden column and recalculate all columns width
	 * @param {Boolean} hidden (optional) true to hide a column (defaults to true)
	 */
	setHidden: function (hidden) {
		if (undefined === hidden || true === hidden) {
			if (this.isHidden()) {
				return;
			}
			this.setWidth('auto');
		} else {
			this.setWidth(this.model.get('width'));
		}
		this.parentWidget.updateColumnsWidth();
	},

	/**
	 * Render Drop spacer for insertion in container relative defined widget
	 * @param {String} insertPlace Available values top|right|bottom|left
	 * @param {Sb.widget.Widget} widget
	 */
	renderDropSpacer: function (insertPlace, widget) {
		var ddSpacer = Sb.Editor.siteView.ddSpacer;

		this.parentWidget.columns.each(function (columnWidget) {
			columnWidget.dt.setPadding(-10, 1, -10, 1);
		});

		if (['top', 'bottom'].indexOf(insertPlace) != -1) {
			Ext.each(['left', 'right'], function (place) {
				var columnInSide = this.parentWidget.getColumnSide(this, place);
				if (columnInSide
					&& columnInSide.isDragWidgetIsOnlyOneInConatainer()
					&& !columnInSide.isHidden()) {
						columnInSide.setHidden(true);
				}
			}, this);
			this.parentWidget.updateColumnsWidth();
			Sb.widget.ColumnContainer.superclass.renderDropSpacer.call(this, insertPlace, widget);
			return;
		}

		var columnInSide = this.parentWidget.getColumnSide(this, insertPlace);
		if (columnInSide) {
			if (columnInSide.isHidden()) {
				this.parentWidget.resetColumnWidth();
				this.refreshCache();
				return;
			}
			if (columnInSide.isDragWidgetIsOnlyOneInConatainer()) {
				this.refreshCache();
				return;
			}
		}

		this.removeDropSpacer();
		var	html = '<td class="widget-columns-column sb-widget-columns-column-ddSpacer">' + this.getDdSpacerHtml() + '</td>',
			htmlSplit = '<td class="widget-columns-split widget-columns-column"></td>',
			place = (insertPlace == 'left') ? 'beforeBegin' : 'afterEnd',
			columnDomEl = Ext.DomHelper.insertHtml(place, this.el.dom, html),
			columnEl = Ext.get(columnDomEl),
			splitWidth = 4; // Width of two splitters width each of them by 2px
		this.parentWidget.updateColumnsWidth();

		if (Sb.Editor.model.isRtl()) {
			insertPlace = Sb.Utils.swapLeftAndRight(insertPlace);
		}
		var columnWidthInPx = columnEl.getWidth(),
			leftDtPadding = (insertPlace == 'left') ? columnWidthInPx + splitWidth : 0,
			rightDtPadding = (insertPlace == 'right') ? columnWidthInPx + splitWidth : 0;
		this.dt.setPadding(-10, rightDtPadding, -10, leftDtPadding);

		Sb.widget.Container.nodes.push(
			columnDomEl,
			Ext.DomHelper.insertHtml(place, this.el.dom, htmlSplit)
		);
		this.refreshCache();
	},

	/**
	 * Checks if we can drop column
	 *
	 * @param {Ext.dd.DragSource} source
	 * @param {Event} e
	 * @return {Boolean}
	 */
	isDroppable: function (source, e) {
		if (!source.dragData.widgetType.isDroppableToColumn()) {
			return false;
		}
		return this.parentWidget.ownerCt.isDroppable(source, e);
	},

	/**
	 * Handler that one of drag the child widget was dropped in invalid target
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onWidgetInvalidDrop: function (dragSource) {
		Sb.widget.ColumnContainer.superclass.onWidgetInvalidDrop.call(this, dragSource);
		this.parentWidget.resetColumnWidth();
	},

	/**
	 * Return that a current drug source a widget and it is only on from this container
	 * @returns {Boolean}
	 */
	isDragWidgetIsOnlyOneInConatainer: function () {
		var dragSource = Ext.dd.DragDropMgr.dragCurrent;
		return dragSource.dragData.widget &&
			this.model.widgets.indexOf(dragSource.dragData.widget.model) != -1 &&
			this.model.widgets.getCount() == 1;
	},

	/**
	 * The function calls when source is move out the container.
	 * @param {Ext.dd.DragSource} source
	 */
	onNotifyOut: function (source) {
		if (this.isDragWidgetIsOnlyOneInConatainer()) {
			this.setHidden(true);
		}
		this.parentWidget.updateColumnsWidth();
		this.dt.setPadding(-10, 1, -10, 1);
	},

	/**
	 * Calls once when dragged item has been dropped in side relative widget in containers
	 * @param {Ext.dd.DragSource} dragSource
	 */
	onDropSide: function (dragSource) {
		var position = this.model.ownerWidget.widgets.indexOf(this.model);
		if (dragSource.dragData.place == 'right') {
			position++;
		}

		this.model.ownerWidget.addColumn(position, dragSource.dragData.widget ?
			dragSource.dragData.widget.model :
			dragSource.dragData.widgetType);
	}
});