/**
 * @class Sb.widget.Columns
 * @extends Sb.widget.Widget
 * Class for Columns Widget view
 * @xtype columns_widget
 */
Ext.define('Sb.widget.Columns', {
	extend: 'Sb.widget.Widget',
	xtype: 'columns_widget',

	cls : 'sb-widget-columns',
	overable: false,
	removeConfirmation: false,

	hasVisibilityControl: false,

	initEvents: function () {
		Sb.widget.Columns.superclass.initEvents.call(this);

		this.mon(this.model.widgets, {
			scope: this,
			add: this.refresh,
			remove: this.refresh
		});
	},

	onContentElementClick: Ext.emptyFn,

	onRender: function () {
		Sb.widget.Columns.superclass.onRender.apply(this, arguments);

		this.columnsTable = this.widgetContentEl.createChild({
			tag: 'table',
			cls: 'widget-columns-table',
			cn: {
				tag: 'tr'
			}
		});
		this.columns = new Ext.util.MixedCollection(false, function (columnWidget) {
			return columnWidget.model.get('uuid');
		});
		this.splits = new Ext.util.MixedCollection();
		this.refresh();
	},

	refresh: function () {
		var defaults = {};
		if (this.clickable === false) {
			// not misprint, temporary solution for columns in blog post teaser
			defaults.clickable = this.clickable;
			defaults.overable = this.clickable;
		}

		this.clearColumns();

		this.model.widgets.each(function (modelColumn, index, count) {
			this.columns.add(new Sb.widget.ColumnContainer({
				model		: modelColumn,
				renderTo	: this.columnsTable.first().first(),
				defaults	: defaults,
				parentWidget: this
			}));

			if (index < count - 1) {
				var split = this.columnsTable.first().first().createChild({
					tag: 'td',
					cls: 'widget-columns-split widget-columns-column',
					cn: [{
						cls: 'widget-columns-resizer-handler'
					}, {
						cls: 'widget-columns-split-vertical-border'
					}]
				});

				if (this.clickable) {
					split.on('mouseenter', function () {
						if (!split.hasClass('sb-widget-columns-split-move')) {
							split.first().setTop(Math.round(split.getHeight() / 2 - 15));
						}
					});
					split.dd = new Ext.dd.DDProxy(split, 'ColumnsSplit');
					split.dd.setOuterHandleElId(split);
					split.dd.b4StartDrag = this.onSplitBeforeMove.createDelegate(this, [split]);
					split.dd.onDrag = this.onSplitMove.createDelegate(this, [split]);
					split.dd.endDrag = this.onSplitApply.createDelegate(this, [split]);
				}
				this.splits.add(split);
			}
		}, this);
	},

	onSplitApply: function (split) {
		Ext.getBody().setStyle({cursor: 'auto'});
		split.removeClass('sb-widget-columns-split-move');
		this.model.onSaveColumnsWidth();
		this.columns.each(function (columnWidget) {
			columnWidget.hideInfo();
		});
		this.doResize();
	},

	onSplitBeforeMove: function (split) {
		Ext.getBody().setStyle({cursor: 'col-resize'});
		split.addClass('sb-widget-columns-split-move');

		split.leftColumn = Ext.getCmp(split.prev().id);
		split.leftColumnWidth = split.leftColumn.getEl().getWidth();

		split.rightColumn = Ext.getCmp(split.next().id);
		split.rightColumnWidth = split.rightColumn.getEl().getWidth();

		split.sum = split.rightColumnWidth + split.leftColumnWidth;
		this.columns.each(function (columnWidget) {
			columnWidget.showInfo();
		});
	},

	onSplitMove: function (split) {
		var newLeftWidth = split.leftColumnWidth +
				(split.dd.lastPageX - split.dd.startPageX) * (Sb.Editor.model.isRtl() ? -1 : 1),
			width = 0;
		this.columns.each(function (columnWidget, index) {
			width += columnWidget.getEl().getWidth();
		});
		var widthInPercent = Ext.util.Format.round(newLeftWidth * 100 / width, 3),
			min = 5,
			max = parseFloat(split.leftColumn.model.get('width')) +
				parseFloat(split.rightColumn.model.get('width')) - min;

		widthInPercent = Math.min(Math.max(widthInPercent, min), max);
		this.model.setColumnWidth(split.leftColumn.model, widthInPercent, false);
		this.columns.each(function (columnWidget) {
			columnWidget.showInfo();
		});
	},

	/**
	 * Reset column width
	 */
	resetColumnWidth: function () {
		this.columns.each(function (columnWidget, position) {
			columnWidget.setWidth(columnWidget.model.get('width'));
		});
	},

	/**
	 * Update width of columns
	 */
	updateColumnsWidth: function () {
		var hiddenColumn = this.getDisabledColumn(),
			enabledColumnsWidth = 0;

		if (hiddenColumn) {
			this.getEnabledColumns().each(function (columnWidget) {
				enabledColumnsWidth += parseFloat(columnWidget.getEl().getStyle('width'));
			});

			if (Ext.util.Format.round(enabledColumnsWidth, 0) != 100) {
				var disabledColumnWidth = parseFloat(hiddenColumn.model.get('width')),
					delta = Ext.util.Format.round(disabledColumnWidth / this.getEnabledColumns().getCount(), 3),
					lastDelta = disabledColumnWidth - delta * (this.getEnabledColumns().getCount() - 1),
					position = this.columns.getCount() - (this.columns.last() === hiddenColumn ? 2 : 1);

				this.getEnabledColumns().each(function (columnWidget, index) {
					var oldWidth = parseFloat(columnWidget.model.get('width')),
						deltaWidth = index == position ? lastDelta : delta,
						newWidth = oldWidth + deltaWidth;
					columnWidget.setWidth(newWidth + '%');
				});
			}
		}

		var newColumnEl = this.columnsTable.child('td.sb-widget-columns-column-ddSpacer');
		if (newColumnEl) {
			var defaultColumnWidth = Ext.util.Format.round(100 / (this.getEnabledColumns().getCount() + 1), 0);
			newColumnEl.setWidth(defaultColumnWidth + '%');

			enabledColumnsWidth = 0;
			this.getEnabledColumns().each(function (columnWidget) {
				enabledColumnsWidth += parseFloat(columnWidget.getEl().getStyle('width'));
			}, this);

			if (Ext.util.Format.round(enabledColumnsWidth, 0) == 100) {
				this.getEnabledColumns().each(function (columnWidget) {
					var oldWidth = parseFloat(columnWidget.getEl().getStyle('width')),
						deltaWidth = Ext.util.Format.round(oldWidth * defaultColumnWidth / 100, 3),
						newWidth = oldWidth - deltaWidth;
					columnWidget.setWidth(newWidth + '%');
				}, this);
			}
		}

		if (!hiddenColumn && !newColumnEl) {
			this.resetColumnWidth();
		}
	},

	/**
	 * Disable defined column and updated other columns width
	 * @param {Sb.widget.ColumnContainer} columnWidget
	 */
	disableColumn: function (columnWidget) {
		columnWidget.setHidden(true);
	},

	/**
	 * Return hidden column
	 * @returns {Sb.widget.ColumnContainer}|null
	 */
	getDisabledColumn: function () {
		var columns = this.columns.filterBy(function (columnWidget) {
			return columnWidget.isHidden();
		});
		return columns.getCount() ? columns.item(0) : null;
	},

	/**
	 * Return enabled columns
	 * @returns {Ext.util.MixedCollection}
	 */
	getEnabledColumns: function () {
		return this.columns.filterBy(function (columnWidget) {
			return !columnWidget.isHidden();
		});
	},

	/**
	 * Return column in side relative defined column and place if exist
	 * @param {Sb.widget.ColumnContainer} columnWidget
	 * @param {String} place Allow place is left|right
	 * @returns {Sb.widget.ColumnContainer}|NULL
	 */
	getColumnSide: function (columnWidget, place) {
		if (['left', 'right'].indexOf(place) == -1) {
			return null;
		}
		var position = this.columns.indexOf(columnWidget) + (place == 'left' ? -1 : 1);
		if (this.columns.item(position) instanceof Sb.widget.ColumnContainer) {
			return this.columns.item(position);
		}
		return null;
	},

	onDestroy: function () {
		if (this.rendered) {
			this.clearColumns();
		}
		Sb.widget.Columns.superclass.onDestroy.call(this);
	},

	clearColumns: function () {
		this.columns.each(function (columnWidget) {
			columnWidget.destroy();
		});
		this.columns.clear();

		this.splits.each(function (split) {
			if (split.dd) {
				split.dd.destroy();
			}
			split.remove();
		});
		this.splits.clear();
	},

	doResize: function () {
		this.callParent();
		this.columns.each(function(item){
			item.doResize();
		}, this);
	},

	startEdit: Ext.emptyFn
});