/**
 * @class Sb.widget.HeaderEditPanel
 * @extends Sb.widget.WidgetEditPanel
 * Class for header widget edit panel
 * @constructor
 * @param {Object} config The config object
 */
Ext.define('Sb.widget.HeaderEditPanel', {
	extend: 'Sb.widget.WidgetEditPanel',

	/**
	 * Header elements checkbox group
	 * @type Ext.form.CheckboxGroup
	 * @property headerElements
	 */
	/**
	 * Image list panel
	 * @type Ext.Panel
	 * @property imageListPanel
	 */
	/**
	 * Image size panel
	 * @type Ext.Panel
	 * @property imageSizePanel
	 */

	/**
	 * @cfg {Number} imageListPageSize Image per page in images list.
	 */
	imageListPageSize: 10,

	/**
	 * Wrapper for edit panel items
	 */
	initComponent: function () {
		Ext.apply(this, {
			width: 453,
			cls: 'sb-widget-edit-panel-header',
			layout: 'column',
			defaults: {
				layout: 'form',
				labelAlign: 'top'
			},
			items: [
				this.createImageListPanel(),
				this.createHeaderElementsPanel()
			]
		});

		Sb.widget.HeaderEditPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Create image list panel
	 *
	 * @return Object
	 */
	createImageListPanel: function () {
		return {
			ref: 'imageListPanel',
			columnWidth: 0.62,
			items: [{
				ref: 'imageList',
				xtype: 'sbheaderselectbox',
				listeners: {
					scope: this,
					beforecollapse: function (selectbox) {
						// Prevent collapsing the list when removal confirmation dialog is visible
						if (selectbox.dialog && selectbox.dialog.isVisible()) {
							return false;
						}
					},
					collapse: function (selectbox) {
						selectbox.header.searchField.setValue('');
					},
					select: this.onImageSelect,
					deleteheader: function (record) {
						var imagePath = Sb.model.HeaderWidget.buildImagePath(record.get('name'));
						if (this.widget.memento.imagePath === imagePath) {
							delete this.widget.memento.imagePath;
							delete this.widget.memento.imageName;
							delete this.widget.memento.imageTime;
							delete this.widget.memento.flashWidth;
							delete this.widget.memento.flashHeight;
						}
					}
				}
			}, {
				ref: 'fitCheckbox',
				xtype: 'checkbox',
				hideLabel: true,
				boxLabel: Sb.WidgetTypeManager.get('header').getTranslation('imageFitSizeTitle'),
				listeners: {
					scope: this,
					afterrender: function (checkbox) {
						var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
							Sb.Editor.Ui.mainWindow, [
								Sb.WidgetTypeManager.get('header').getTranslation('imageFitSizeTooltip')
							]
						);
						checkbox.el.on('mouseenter', fn);
						checkbox.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

						checkbox.labelEl.on('mouseenter', fn);
						checkbox.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
					}
				}
			}]
		};
	},

	/**
	 * Create Header elements panel
	 * Header logo, title, subtitle
	 *
	 * @return Object
	 */
	createHeaderElementsPanel: function () {
		return {
			columnWidth: 0.38,
			cls: 'sb-widget-header-show-elements',
			items: [{
				width: 157,
				html: Sb.WidgetTypeManager.get('header').getTranslation('showElements')
			}, {
				ref: '../headerElements',
				xtype: 'checkboxgroup',
				width: 238,
				columns: 1,
				items: [{
					name: 'logo',
					boxLabel: Sb.WidgetTypeManager.get('header').getTranslation('showLogo'),
					listeners: {
						scope: this,
						check: function (check, checked) {
							this.widget.model.set('logoIsHidden', !checked);
						}
					}
				}, {
					name: 'title',
					boxLabel: Sb.WidgetTypeManager.get('header').getTranslation('showTitle'),
					listeners: {
						scope: this,
						check: function (check, checked) {
							this.widget.model.set('titleIsHidden', !checked);
						}
					}
				}, {
					name: 'subtitle',
					boxLabel: Sb.WidgetTypeManager.get('header').getTranslation('showSubtitle'),
					itemCls: 'sb-editor-panel-header-showsubtitle',
					listeners: {
						scope: this,
						check: function (check, checked) {
							this.widget.model.set('subtitleIsHidden', !checked);
						}
					}
				}, {
					name: 'text',
					boxLabel: Sb.WidgetTypeManager.get('header').getTranslation('showText'),
					listeners: {
						scope: this,
						check: function (check, checked) {
							this.widget.model.set('textIsHidden', !checked);
						}
					}
				}]
			}]
		};
	},

	afterRender: function () {
		Sb.widget.HeaderEditPanel.superclass.afterRender.apply(this, arguments);

		Sb.HeaderManager.loadHeaders();

		Sb.Editor.model.design.on('save', function (design) {
			if (design.presetType == 'special') {
				Sb.HeaderManager.loadSiteHeaders();
			}
		}, this);
	},

	setImageListInitialValue: function () {
		var imageList = this.imageListPanel.imageList,
			imagePath = this.widget.model.get('imagePath'),
			imageName = this.widget.model.get('imageName'),
			imageFormat;

		if (imageName !== '') {
			imageFormat = imagePath === ''
				? Sb.HeaderManager.getHeaderFormat(imageName)
				: imagePath.split('.').slice(-1)[0];
			imageList.setValue(Sb.Config.headersUrl + '/' + imageName + '/preview.' + imageFormat);
		} else {
			imagePath = imagePath.replace('/image_', '/thumbnail_image_');
			imageList.setValue(
				this.widget.model.getResourcesUrl() + '/' + imagePath + '?' + this.widget.model.data.imageTime
			);
		}
	},

	setFitCheckboxVisibility: function () {
		var model = this.widget.model,
			url = model.getResourcesUrl() + '/' + model.get('imagePath'),
			isFlash = Sb.utils.Flash.isFlash(url),
			isCustom = !model.get('imageName');

		this.imageListPanel.fitCheckbox.setVisible(isCustom && !isFlash);
	},

	show: function (widget) {
		var model;

		Sb.widget.HeaderEditPanel.superclass.show.call(this, widget);

		this.setImageListInitialValue();
		this.bindModel({
			imageFit: this.imageListPanel.fitCheckbox
		});

		model = this.widget.model;
		this.headerElements.setValue({
			logo: !model.get('logoIsHidden'),
			title: !model.get('titleIsHidden'),
			subtitle: !model.get('subtitleIsHidden'),
			text: !model.get('textIsHidden')
		});

		this.setFitCheckboxVisibility();
	},

	onImageSelect: function (combo, record) {
		this.widget.model.unserialize(this.prepareData(record));
		this.setFitCheckboxVisibility();
	},

	prepareData: function (record) {
		var data = {},
			height;

		if (Sb.utils.Flash.isFlash(record.data.path)) {
			height = record.data.flashHeight;
			data = {
				height: height,
				flashWidth: record.data.flashWidth,
				flashHeight: height
			};
		}
		if (record.data.isCustom) {
			data.imageName = '';
			data.imagePath = Sb.model.HeaderWidget.buildImagePath(record.data.name);
			data.imageTime = record.data.time;
		} else {
			data.imageName = record.data.name;
			data.imagePath = '';
		}

		// Clear smart resize data
		data.areas = record.data.areas;
		data.imageSize = record.data.imageSize;
		return data;
	}
});