/**
 * @class Sb.widget.SiteLogo
 * @extends Sb.widget.Widget
 * Class for site logo widget
 * @constructor
 * @param {Object} config The config object
 * @xtype site_logo_widget
 */
Ext.define('Sb.widget.SiteLogo', {
	extend: 'Sb.widget.Widget',
	xtype: 'site_logo_widget',

	cls: 'sb-widget-site-logo',
	isProportional: true,
	hasResizer: false,

	onRender: function () {
		Sb.widget.SiteLogo.superclass.onRender.apply(this, arguments);

		this.imageUrl = this.model.getResourcesUrl() + '/' + Sb.Editor.model.get('logoPath');
		this.updateImage();

		var isProportional = this.model.get('width') && this.model.get('height')
			? !!this.model.get('proportional')
			: true;

		this.setProportional(isProportional);
		this.updateAlignment(this.model.get('alignment') || 'left');

		this.mon(Sb.Editor.model, 'propertychange', function (property) {
			if (property == 'logoPath') {
				var logoPath = Sb.Editor.model.get('logoPath');

				this.updateSize({width: '', height: ''});
				this.updateImage(this.model.getResourcesUrl() + '/' + logoPath);
				this.model.save();
			}
		}, this);
	},

	onStartEdit: function () {
		var editPanel = this.getEditPanel();
		this.startImage = this.imageUrl;
		Sb.widget.SiteLogo.superclass.onStartEdit.call(this);
		this.mon(editPanel, 'imagechange', this.onImageChange, this);
		this.mon(editPanel, 'sizechange', this.onSizeChange, this);
		this.mon(editPanel, 'alignmentchange', this.onAlignmentChange, this);
		this.mon(editPanel, 'proportionschange', this.onProportionsChange, this);
		this.updateEditPanel();
		this.showImageResizer();
	},

	onCompleteEdit: function () {
		var editPanel = this.getEditPanel();
		this.mun(editPanel, 'imagechange', this.onImageChange, this);
		this.mun(editPanel, 'sizechange', this.onSizeChange, this);
		this.mun(editPanel, 'alignmentchange', this.onAlignmentChange, this);
		this.mun(editPanel, 'proportionschange', this.onProportionsChange, this);
		if (Sb.Editor.model.get('logoPath')
			!= this.startImage.replace(new RegExp('^' + this.model.getResourcesUrl() + '/' + '(.*)'), '$1')) {
			Sb.Editor.model.save();
		}
		this.model.unserialize({
			proportional: this.isProportional,
			alignment: this.alignment
		});
		Sb.widget.SiteLogo.superclass.onCompleteEdit.apply(this, arguments);
		this.hideImageResizer();
	},

	onCancelEdit: function () {
		this.updateImage(this.startImage);
		this.setProportional(this.memento.proportional);
		this.updateSize({width: this.memento.width, height: this.memento.height});
		this.updateAlignment(this.memento.alignment);
		Sb.widget.SiteLogo.superclass.onCancelEdit.apply(this);
		this.hideImageResizer();
	},

	getImageSize: function () {
		if (this.model.get('width') && this.model.get('height')) {
			return {width: this.model.get('width'), height: this.model.get('height')};
		}

		if (this.imageEl) {
			return this.imageEl.getSize();
		}

		return {width: 0, height: 0};
	},

	/**
	 * Sets this Image's src
	 * @param {String} [url] The url to an image
	 */
	updateImage: function (url) {
		this.imageUrl = url || this.imageUrl || Ext.BLANK_IMAGE_URL;
		this.hideImageResizer();
		if (this.imageWrapEl) {
			Ext.destroy(this.imageWrapEl);
		}

		this.imageWrapEl = this.widgetContentEl.createChild({
			cls: 'sb-widget-site-logo-wrapper'
		});
		if (Sb.utils.Flash.isFlash(this.imageUrl)) {
			this.imageEl = this.createFlashEl();
			this.onImageLoad();
		} else {
			this.imageEl = this.imageWrapEl.createChild({
				tag: 'img',
				alt: ''
			});
			this.imageEl.on('load', this.onImageLoad, this, {single: true});
			this.imageEl.set({'src': this.imageUrl});
		}
		this.originalSize = null;
		var logoPath = this.imageUrl.replace(new RegExp('^' + this.model.getResourcesUrl() + '/' + '(.*)'), '$1');
		Sb.Editor.model.set('logoPath', logoPath);
	},

	createFlashEl: function () {
		return this.imageWrapEl.createChild({
			tag: 'div',
			html: Sb.utils.Flash.createCode(
				this.imageUrl,
				this.model.get('width'),
				this.model.get('height')
			)
		});
	},

	/**
	 * Sets this Image's size
	 * @param {Object} size like {width, height}
	 */
	updateSize: function (size) {
		if (Sb.utils.Flash.isFlash(this.imageUrl) && (!size.width || !size.height)) {
			size = Sb.utils.Flash.getSize(this.imageUrl);
		}

		this.model.unserialize(size);

		if (!this.imageEl) {
			return;
		}

		if (size.width && size.height) {
			Sb.utils.Flash.isFlash(this.imageUrl)
				? Sb.utils.Flash.setObjectSize(this.imageEl, size)
				: this.imageEl.setSize('100%', '100%');
		} else {
			size = this.getImageSize();
			this.imageEl.setSize('', '');
		}
		this.imageWrapEl.setSize(size.width, size.height);
		this.updateEditPanel();
	},

	updateAlignment: function (alignment) {
		if (alignment == this.alignment) {
			return;
		}

		this.widgetContentEl.setStyle({textAlign: alignment});
		this.alignment = alignment;
		this.updateEditPanel();
	},

	/**
	 * Set image to keep original aspect ratio
	 * @param {Boolean} isProportional
	 */
	setProportional: function (isProportional) {
		if (isProportional == this.isProportional) {
			return;
		}

		this.isProportional = isProportional;
		this.updateEditPanel();
	},

	/**
	 * @param {Boolean} [reanchor]
	 */
	initEditPanel: function (reanchor) {
		var editPanel = this.getEditPanel();

		if (Sb.utils.Flash.isFlash(this.imageUrl)) {
			this.originalSize = Sb.utils.Flash.getSize(this.imageUrl);
			editPanel.originalImageSize = this.originalSize;
			this.updateEditPanel(reanchor);
		} else {
			Sb.Utils.getImageOriginalSize(this.imageUrl, function (originalSize) {
				this.originalSize = originalSize;
				editPanel.originalImageSize = this.originalSize;
				this.updateEditPanel(reanchor);
			}.createDelegate(this));
		}
	},

	/**
	 * @param {Boolean} [reanchor=true]
	 */
	updateEditPanel: function (reanchor) {
		reanchor = undefined === reanchor || reanchor;
		if (Sb.Editor.Ui.editComponent != this || !this.getEditPanel()) {
			return;
		}

		if (!this.originalSize) {
			this.initEditPanel(reanchor);
			return;
		}

		var editPanel = this.getEditPanel();

		editPanel.updateFields({
			imageSize: this.getImageSize(),
			isProportional: this.isProportional,
			alignment: this.alignment
		});

		if (reanchor) {
			editPanel.reanchor();
		}
	},

	showImageResizer: function () {
		this.imageResizer = new Ext.Resizable(this.imageWrapEl.id, {
			handles: 'all',
			pinned: true,
			minHeight: 20,
			minWidth: 20,
			maxHeight: 1000,
			maxWidth: 1000,
			listeners: {
				scope: this,
				beforeresize: this.onBeforeMouseResize,
				resize: this.onMouseResize
			}
		});
	},

	hideImageResizer: function () {
		if (!this.imageResizer) {
			return;
		}

		this.imageWrapEl.removeClass('x-resizable-pinned');
		this.imageResizer.destroy();
	},

	onImageChange: function (editPanel, fileName) {
		this.model.unserialize({width: '', height: ''});
		this.updateImage(this.model.getResourcesUrl() + '/' + Sb.Config.path.attachmentsFolder + '/Logo/' + fileName);
	},

	onSizeChange: function (editPanel, size) {
		this.updateSize(size);
	},

	onAlignmentChange: function (editPanel, alignment) {
		this.updateAlignment(alignment);
	},

	onImageLoad: function () {
		this.updateSize({width: this.model.get('width'), height: this.model.get('height')});

		if (Sb.Editor.Ui.editComponent == this) {
			var editPanel = this.getEditPanel();

			this.showImageResizer();
			editPanel.show(this);
			this.updateEditPanel();
		}
	},

	onProportionsChange: function (editPanel, isProportional) {
		this.setProportional(isProportional);
	},

	onBeforeMouseResize: function () {
		this.lastPosition = this.imageWrapEl.getPositioning();
	},

	onMouseResize: function (resizer, width, height) {
		this.setProportional(false);
		this.imageWrapEl.setPositioning(this.lastPosition);
		this.updateSize({width: width, height: height});
	}
});