/**
 * Base class for all widget style panels
 *
 * @class Sb.widget.WidgetStylePanel
 * @extends Sb.editor.component.BaseStylePanel
 */
Ext.define('Sb.widget.WidgetStylePanel', {
	extend: 'Sb.editor.component.BaseStylePanel',

	initComponent: function () {
		Ext.apply(this, {
			autoHeight: true,
			layout: 'form',
			labelAlign: 'right',
			labelWidth: 130,
			items: [{
				ref: 'paddingField',
				xtype: 'expander',
				fieldLabel: Sb.Locale.get('padding'),
				minValue: 0,
				listeners: {
					scope: this,
					expand: this.onControlHeightChange,
					collapse: this.onControlHeightChange
				}
			}, {
				ref: 'marginField',
				xtype: 'expander',
				fieldLabel: Sb.Locale.get('margins'),
				itemCls: 'sb-bottom-border',
				listeners: {
					scope: this,
					expand: this.onControlHeightChange,
					collapse: this.onControlHeightChange
				}
			}, {
				ref: 'shadowField',
				xtype: 'sbshadowselectbox',
				itemCls: 'sb-bottom-border',
				fieldLabel: Sb.Locale.get('shadows')
			}, {
				ref: 'cornersField',
				xtype: 'expander',
				mode: 'corner',
				fieldLabel: Sb.Locale.get('corners')
			}, {
				ref: 'backgroundColorField',
				xtype: 'colordropdown',
				itemId: 'color-dropdown-' + this.id,
				itemCls: 'sb-bottom-border',
				allowBackground: true,
				fieldLabel: Sb.Locale.get('background'),
				allowBlank: true,
				allowOpacity: true,
				allowColorPalette: true,
				width: 128,
				listeners: {
					scope: this,
					backgroundchange: this.onBackgroundChange
				}
			}, {
				ref: 'textColorField',
				xtype: 'colordropdown',
				itemId: this.id + '-text-color',
				fieldLabel: Sb.Locale.get('textColorLabel'),
				allowStroke: true,
				allowInherit: true,
				allowColorPalette: true,
				width: 128,
				listeners: {
					scope: this,
					backgroundchange: this.onBackgroundChange
				}
			}, {
				ref: 'linkColorField',
				xtype: 'colordropdown',
				itemId: this.id + '-link-color',
				hidden: true,
				fieldLabel: Sb.Locale.get('linkColorLabel'),
				allowStroke: true,
				allowInherit: true,
				allowColorPalette: true,
				width: 128,
				listeners: {
					scope: this,
					backgroundchange: this.onBackgroundChange
				}
			}, {
				ref: 'h1ColorField',
				xtype: 'colordropdown',
				itemId: this.id + '-h1-color',
				hidden: true,
				fieldLabel: Sb.Locale.get('h1ColorLabel'),
				allowStroke: true,
				allowInherit: true,
				allowColorPalette: true,
				width: 128,
				listeners: {
					scope: this,
					backgroundchange: this.onBackgroundChange
				}
			}, {
				ref: 'h2ColorField',
				xtype: 'colordropdown',
				itemId: this.id + '-h2-color',
				hidden: true,
				allowStroke: true,
				fieldLabel: Sb.Locale.get('h2ColorLabel'),
				allowInherit: true,
				allowColorPalette: true,
				width: 128,
				listeners: {
					scope: this,
					backgroundchange: this.onBackgroundChange
				}
			}, {
				ref: 'moreColorsField',
				xtype: 'displayfield',
				cls: 'sb-widget-style-panel-more-colors sb-widget-style-panel-more-colors-collapsed',
				autoCreate: {
					tag: 'a',
					href: '#'
				},
				value: Sb.Locale.get('showMoreColors'),
				listeners: {
					scope: this,
					afterrender: function(field) {
						field.el.on('click', function() {
							field.el.toggleClass('sb-widget-style-panel-more-colors-collapsed');
							field.setValue(Sb.Locale.get(
								field.el.hasClass('sb-widget-style-panel-more-colors-collapsed')
									? 'showMoreColors' : 'showLessColors'
							));
							this.updateColorsVisibility();
						}, this);
					}
				}
			}]
		});

		Sb.widget.WidgetStylePanel.superclass.initComponent.apply(this, arguments);
	},

	show: function (widget) {
		var styles = widget.model.get('styles'),
			backgroundType = Sb.Utils.getBackgroundType(styles.background).value;

		this.widget = widget;

		Sb.widget.WidgetStylePanel.superclass.show.call(this, widget);

		this.bindModel({
			padding: this.paddingField,
			margin: this.marginField,
			borderRadius: this.cornersField,
			boxShadow: this.shadowField,
			backgroundColor: this.backgroundColorField,
			textColor: this.textColorField,
			textStroke: this.textColorField.menu.strokeField,
			linkColor: this.linkColorField,
			linkStroke: this.linkColorField.menu.strokeField,
			h1Color: this.h1ColorField,
			h1Stroke: this.h1ColorField.menu.strokeField,
			h2Color: this.h2ColorField,
			h2Stroke: this.h2ColorField.menu.strokeField
		});

		this.cascade(function (cmp) {
			if (cmp.getXType() === 'expander') {
				cmp.collapse();
			}
		});

		this.backgroundColorField.setBackgroundUploadParams(Sb.Config.baseUrl + '/Upload/Background/', {});
		this.backgroundColorField.setBackgroundValue({
			type: backgroundType,
			gradient: backgroundType === 'gradient' ? styles.background : 'background10-black',
			pattern: backgroundType === 'pattern' ? styles.background : 'background1',
			position: styles.backgroundPosition || 'top left',
			stretch: backgroundType === 'pattern' ? styles.backgroundStretch : 'tile',
			opacity: styles.backgroundOpacity || 0
		});

		var computedStyles = this.widget.model.ownerWidget.getComputedStyles();
		this.textColorField.setParentColor(computedStyles.textColor);
		this.linkColorField.setParentColor(computedStyles.linkColor);
		this.h1ColorField.setParentColor(computedStyles.h1Color);
		this.h2ColorField.setParentColor(computedStyles.h2Color);

		this.updateColorsVisibility();
	},

	updateColorsVisibility: function () {
		var widgetType = this.widget.getType(),
			allowedColors = widgetType.allowedColors ? widgetType.allowedColors.split(',') : [],
			moreColorsCollapsed = this.moreColorsField.el.hasClass('sb-widget-style-panel-more-colors-collapsed'),
			n = 0;

		['text', 'link', 'h1', 'h2'].forEach(function (type) {
			if (allowedColors.indexOf(type) == -1) {
				this[type + 'ColorField'].setVisible(false);
			} else {
				this[type + 'ColorField'].setVisible(!n || allowedColors.length <= 2 || !moreColorsCollapsed);
				n++;
			}
		}, this);
		this.moreColorsField.setVisible(allowedColors.length > 2);
		this.backgroundColorField.itemCt[allowedColors.length ? 'addClass' : 'removeClass']('sb-bottom-border');
	},

	/**
	 * Binds fields of the panel to model, unversally integrating change events.
	 *
	 * @param {Array|Object} fieldsMapping Mapping of fields to model. Could be just array or object modelKey => fieldName.
	 * @param {Sb.model.Node} [model] Model for binding. Defaults to the current Widget.
	 */
	bindModel: function (fieldsMapping, model) {
		model = model || this.widget.model;
		var fields = {};

		if (Ext.isString(fieldsMapping)) {
			fieldsMapping = [fieldsMapping];
		}
		if (Ext.isArray(fieldsMapping)) {
			Ext.each(fieldsMapping, function (item) {
				fields[item] = item;
			});
		} else {
			fields	= fieldsMapping;
		}

		Ext.iterate(fields, function (modelKey, field) {
			var	fieldObj	= Ext.isObject(field) ? field : this[field],
				fieldXType = fieldObj.getXType(),
				eventName	= 'change',
				getValueFn	= 'getValue',
				setValueFn	= 'setValue',
				changeFn	= function () {
					var styles = {};
					if (this.isValid()) {
						styles[modelKey] = this[getValueFn]();
						model.set('styles', styles);
					}
				};

			if (fieldXType === 'sbshadowselectbox') {
				eventName = 'select';
			} else if (fieldXType === 'checkbox') {
				eventName = 'check';
			}

			fieldObj[setValueFn](model.get('styles')[modelKey]);
			fieldObj.on(eventName, changeFn);
			this.on('hide', function () {
				fieldObj.un(eventName, changeFn);
			}, this, {single: true});
		}, this);
	},

	onControlHeightChange: function () {
		this.reanchor(false);
	},

	onBackgroundColorChange: function (field, color) {
		this.widget.model.set('styles', {backgroundColor: color});
	},

	onBackgroundChange: function (colormenu, background) {
		var styles = {
				backgroundOpacity: background.opacity
			};

		if (background.type == 'gradient') {
			styles.background = background.gradient;
		}

		if (background.type == 'pattern') {
			styles.background = background.pattern;
			styles.backgroundPosition = background.position;
			styles.backgroundStretch = background.stretch;
		}

		if (background.type == 'none') {
			styles.background = '';
		}

		this.widget.model.set('styles', styles);
	}
});