/*!
 * Ext JS Library 3.2.1
 * Copyright(c) 2006-2010 Ext JS, Inc.
 * licensing@extjs.com
 * http://www.extjs.com/license
 */
/**
 * @class Ext.ux.form.FileUploadField
 * @extends Ext.form.TextField
 * Creates a file upload field.
 * @xtype fileuploadfield
 */
Ext.define('Ext.ux.form.FileUploadField', {
	extend: 'Ext.form.TextField',
	xtype: 'fileuploadfield',

	/**
	 * @cfg {String} buttonText The button text to display on the upload button (defaults to
	 * 'Browse...').  Note that if you supply a value for {@link #buttonCfg}, the buttonCfg.text
	 * value will be used instead if available.
	 */
	buttonText: 'Browse...',
	/**
	 * @cfg {Boolean} buttonOnly True to display the file upload field as a button with no visible
	 * text field (defaults to false).  If true, all inherited TextField members will still be available.
	 */
	buttonOnly: false,
	/**
	 * @cfg {Number} buttonOffset The number of pixels of space reserved between the button and the text field
	 * (defaults to 3).  Note that this only applies if {@link #buttonOnly} = false.
	 */
	buttonOffset: 3,
	/**
	 * @cfg {Object} customButtonCfg Config of custom button (Ext.component config)
	 */
	customButtonCfg: null,
	/**
	 * @cfg {Object} buttonCfg A standard {@link Ext.Button} config object.
	 */
	/**
	 * @cfg {Boolean} multiple True to allow multiple file upload (defaults to false).
	 */
	multiple: false,

	/**
	 * @cfg {String} accept Filter allowed files by mime/type (defaults to empty string).
	 */
	accept: '',

	// private
	readOnly: true,

	/**
	 * @hide
	 * @method autoSize
	 */
	autoSize: Ext.emptyFn,

	// private
	initComponent: function () {
		Ext.ux.form.FileUploadField.superclass.initComponent.call(this);

		this.addEvents(
			/**
			 * @event fileselected
			 * Fires when the underlying file input field's value has changed from the user
			 * selecting a new file from the system file selection dialog.
			 * @param {Ext.ux.form.FileUploadField} this
			 * @param {String} value The file value returned by the underlying file input field
			 */
			'fileselected'
		);
	},

	// private
	onRender : function (ct, position) {
		Ext.ux.form.FileUploadField.superclass.onRender.call(this, ct, position);

		this.wrap = this.el.wrap({cls: 'x-form-field-wrap x-form-file-wrap'});
		this.el.addClass('x-form-file-text');
		this.el.dom.removeAttribute('name');
		this.createFileInput();

		var btnCfg = Ext.applyIf(this.buttonCfg || {}, {
			text: this.buttonText
		});
		this.button = this.customButtonCfg
			? new Ext.Component(Ext.apply(this.customButtonCfg, {renderTo: this.wrap}))
			: new Ext.Button(Ext.apply(btnCfg, {
				renderTo: this.wrap,
				cls: 'x-form-file-btn' + (btnCfg.iconCls ? ' x-btn-icon' : '')
		}));

		this.bindListeners();
		this.resizeEl = this.positionEl = this.wrap;
	},

	afterRender: function (c) {
		var width, button;

		Ext.ux.form.FileUploadField.superclass.afterRender.call(this, c);

		if (this.buttonOnly) {
			this.el.hide();
			button = this.button.getEl();
			width = button.getWidth();
			if (Ext.isIE10) {
				width++;
				this.button.getEl().setWidth(width);
			}
			this.wrap.setWidth(width);
		}
	},

	bindListeners: function () {
		this.fileInput.on({
			scope: this,
			mouseenter: function () {
				this.button.addClass(['x-btn-over', 'x-btn-focus']);
			},
			mouseleave: function () {
				this.button.removeClass(['x-btn-over', 'x-btn-focus', 'x-btn-click']);
			},
			mousedown: function () {
				this.button.addClass('x-btn-click');
			},
			mouseup: function () {
				this.button.removeClass(['x-btn-over', 'x-btn-focus', 'x-btn-click']);
				if (Ext.isSafari) {
					var denyClicksMask = Ext.getBody().mask();
					denyClicksMask.setStyle({
						zIndex			: 40000,
						opacity			: 0
					});
					denyClicksMask.on('click', function () {
						Ext.getBody().unmask();
					});
				}
			},
			change: function () {
				if (Ext.isSafari) {
					Ext.getBody().unmask();
				}

				var d = this.fileInput.dom, v = [], i, iMax;

				if (d.files) {
					for (i = 0, iMax = d.files.length; i < iMax; i++) {
						v.push(d.files[i].name);
					}
					v = (v.length === 0) ? [''] : v;
				} else {
					v.push(d.value);
				}
				v = (v.length == 1) ? v[0] : v;

				this.setValue(v);
				this.fireEvent('fileselected', this, v);
			}
		});
	},

	createFileInput : function () {
		var cfg = {
			id			: this.getFileInputId(),
			name		: this.name || this.getId(),
			cls			: 'x-form-file',
			tag			: 'input',
			type		: 'file',
			size		: 1,
			tabindex	: -1,
			accept		: this.accept
		};
		if (this.multiple) {
			cfg.multiple = true;
			cfg.name += '[]';
		}

		this.fileInput = this.wrap.createChild(cfg);
	},

	reset : function (){
		if (this.rendered) {
			this.fileInput.remove();
			this.createFileInput();
			this.bindListeners();
		}
		Ext.ux.form.FileUploadField.superclass.reset.call(this);
	},

	// private
	getFileInputId: function (){
		return this.id + '-file';
	},

	// private
	onResize : function (w, h){
		Ext.ux.form.FileUploadField.superclass.onResize.call(this, w, h);

		this.wrap.setWidth(w);

		if (!this.buttonOnly) {
			w = this.wrap.getWidth() - this.button.getEl().getWidth() - this.buttonOffset;
			this.el.setWidth(w);
		}
	},

	// private
	onDestroy: function (){
		Ext.ux.form.FileUploadField.superclass.onDestroy.call(this);
		Ext.destroy(this.fileInput, this.button, this.wrap);
	},

	onDisable: function (){
		Ext.ux.form.FileUploadField.superclass.onDisable.call(this);
		this.doDisable(true);
	},

	onEnable: function (){
		Ext.ux.form.FileUploadField.superclass.onEnable.call(this);
		this.doDisable(false);

	},

	// private
	doDisable: function (disabled){
		this.fileInput.dom.disabled = disabled;
		this.button.setDisabled(disabled);
	},

	// private
	preFocus : Ext.emptyFn,

	// private
	alignErrorIcon : function (){
		this.errorIcon.alignTo(this.wrap, 'tl-tr', [2, 0]);
	},

	/**
	 * Work only in HTML5
	 *
	 * @return {FileList}
	 */
	getFiles: function () {
		return this.fileInput.dom.files;
	},

	/**
	 * Sets this the upload button's text
	 * @param {String} text The button text
	 */
	setButtonText: function(text) {
		this.buttonText = text;
		if (this.button) {
			this.button.setText(text);
			if (this.buttonOnly) {
				this.wrap.setWidth(this.button.getEl().getWidth());
			}
		}
	}
});

// backwards compat
Ext.form.FileUploadField = Ext.ux.form.FileUploadField;
